@@ -1,3 +1,8 @@
+/*
+ * $Id: optimizer.c 21353 2007-09-18 18:28:31Z paultcochrane $
+ * Copyright (C) 2002-2007, The Perl Foundation.
+ */
+
 /*
  * optimizer.c
  *
@@ -94,18 +99,6 @@ static int branch_cond_loop_swap( PARROT_INTERP,
 static int branch_reorg( PARROT_INTERP, IMC_Unit * unit )
         __attribute__nonnull__(1);
 
-PARROT_WARN_UNUSED_RESULT
-static int check_clone( PARROT_INTERP,
-    NOTNULL(IMC_Unit *unit),
-    NOTNULL(Instruction *ins) )
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(3);
-
-static int clone_remove( PARROT_INTERP, NOTNULL(IMC_Unit *unit) )
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
-
 static int constant_propagation( PARROT_INTERP, IMC_Unit * unit )
         __attribute__nonnull__(1);
 
@@ -230,8 +223,6 @@ optimize(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
     if (IMCC_INFO(interp)->optimizer_level & OPT_CFG) {
         IMCC_info(interp, 2, "optimize\n");
         any = constant_propagation(interp, unit);
-        if (0 && clone_remove(interp, unit)) /* XXX Why is this commented out? */
-            return 1;
         if (used_once(interp, unit))
             return 1;
 #if DO_LOOP_OPTIMIZATION
@@ -1088,6 +1079,7 @@ branch_cond_loop_swap(PARROT_INTERP, IMC_Unit *unit, Instruction *branch,
         if (found) {
             Instruction *tmp;
             SymReg *regs[3], *r;
+            int reg_index;
 
             /* cond_op has 2 or 3 args */
             PARROT_ASSERT(args <= 3);
@@ -1109,8 +1101,12 @@ branch_cond_loop_swap(PARROT_INTERP, IMC_Unit *unit, Instruction *branch,
                 regs[count] = cond->r[count];
             }
 
-            regs[get_branch_regno(cond)] =
-                mk_label_address(interp, str_dup(label));
+            reg_index = get_branch_regno(cond);
+            if (reg_index < 0) {
+                real_exception(interp, NULL, 1,
+                        "Negative branch register address detected");
+            }
+            regs[reg_index] = mk_label_address(interp, str_dup(label));
             tmp = INS(interp, unit, (const char*)neg_op, "", regs, args, 0, 0);
 
             IMCC_debug(interp, DEBUG_OPT1,
@@ -1629,34 +1625,6 @@ loop_optimization(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
 }
 #endif
 
-PARROT_WARN_UNUSED_RESULT
-static int
-check_clone(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(Instruction *ins))
-{
-    SymReg * rl = ins->r[0];
-    SymReg * rr = ins->r[1];
-    if (0 && is_ins_save(interp, unit, ins, rl, CHK_CLONE) &&
-        is_ins_save(interp, unit, ins, rr, CHK_CLONE)) {
-        IMCC_debug(interp, DEBUG_OPT2, "clone %I removed\n", ins);
-        free(ins->op);
-        ins->op = str_dup("set");
-        return 1;
-    }
-    return 0;
-}
-
-static int
-clone_remove(PARROT_INTERP, NOTNULL(IMC_Unit *unit))
-{
-    Instruction *ins;
-    int changes = 0;
-    IMCC_debug(interp, DEBUG_OPT2, "clone_remove\n");
-    for (ins = unit->instructions; ins; ins = ins->next)
-        if (!strcmp(ins->op, "clone"))
-            changes |= check_clone(interp, unit, ins);
-    return changes;
-}
-
 /*
  * Local variables:
  *   c-file-style: "parrot"
