@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2007, The Perl Foundation.
- * $Id: pbc.c 20690 2007-08-19 00:29:46Z chromatic $
+ * Copyright (C) 2002-2007, The Perl Foundation.
+ * $Id: pbc.c 20902 2007-08-29 12:19:05Z paultcochrane $
  */
 
 #include "imc.h"
@@ -503,16 +503,14 @@ fixup_globals(PARROT_INTERP)
                 }
                 if (!s1) {
                     int op, col;
-                    SymReg *nam;
+                    SymReg *nam = mk_const(interp, str_dup(fixup->name),
+                            fixup->type & VT_ENCODED ? 'U' : 'S');
 
                     op = interp->op_lib->op_code("find_name_p_sc", 1);
                     PARROT_ASSERT(op);
 
                     interp->code->base.data[addr] = op;
 
-                    nam = mk_const(interp, str_dup(fixup->name),
-                            fixup->type & VT_ENCODED ? 'U' : 'S');
-
                     if (nam->color >= 0)
                         col = nam->color;
                     else
