@@ -1,4 +1,7 @@
 /*
+ * $Id: pcc.c 21285 2007-09-14 18:47:13Z particle $
+ * Copyright (C) 2003-2007, The Perl Foundation.
+ *
  * pcc.c
  *
  * Parrot calling convention implementation.
@@ -255,6 +258,15 @@ expand_pcc_sub(PARROT_INTERP, NOTNULL(NOTNULL(IMC_Unit *unit)), NOTNULL(NOTNULL(
             strcmp(unit->last_ins->op, "returncc")
        ) {
         Instruction *tmp;
+
+        /* check to make sure the sub is ok before we try to use it */
+        if (!sub) {
+            real_exception(interp, NULL, 1, "NULL sub detected");
+        }
+        if (!sub->pcc_sub) {
+            real_exception(interp, NULL, 1, "NULL sub->pcc_sub detected");
+        }
+
         if (sub->pcc_sub->pragma & P_MAIN) {
             tmp = INS(interp, unit, "end", NULL, regs, 0, 0, 0);
         }
@@ -397,12 +409,12 @@ move_regs(PARROT_INTERP, IMC_Unit * unit,
             SymReg * const rj = j < n ? dest[j] : src[j - n];
             if (ri == rj) {
                 PARROT_ASSERT(j < 255);
-                move_list[i] = j;
+                move_list[i] = (unsigned char)j;
                 goto done;
             }
         }
         PARROT_ASSERT(i < 255);
-        move_list[i] = i;
+        move_list[i] = (unsigned char)i;
 done:
         ;
     }
@@ -551,7 +563,6 @@ expand_pcc_sub_call(PARROT_INTERP, NOTNULL(IMC_Unit *unit), NOTNULL(Instruction
             /*
              * insert set_p_pc with the sub as constant
              */
-            the_sub         = dup_sym(the_sub);
             the_sub->set    = 'p';
             the_sub->usage  = U_FIXUP;
             the_sub->type  &= ~VTADDRESS;
