@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: gcc.pm 20663 2007-08-18 01:37:27Z chromatic $
+# $Id: gcc.pm 20938 2007-08-31 12:32:47Z paultcochrane $
 
 =head1 NAME
 
@@ -104,7 +104,7 @@ sub runstep {
                 . " -Waggregate-return"
                 . " -Winline"
                 . " -Wno-unused"
-                . " -Wnested-externs -Wfloat-equal"
+                . " -Wnested-externs"
                 . ( $maint ? " -Wlarger-than-4096" : "" ),
 
             # others; ones we might like marked with ?
@@ -185,7 +185,7 @@ sub runstep {
                 . " -Wno-div-by-zero"
                 . " -Wno-endif-labels"
                 . " -Werror-implicit-function-declaration"
-                . " -Wfloat-equal"
+                #. " -Wfloat-equal"
                 . " -Wformat"
                 . " -Wformat=2"
                 . " -Wno-format-extra-args"
@@ -280,11 +280,12 @@ sub runstep {
             #3.0 => "-Wformat-nonliteral -Wformat-security -Wpacked "
             #    . "-Wdisabled-optimization -mno-accumulate-outgoing-args "
             #    . "-Wno-shadow -falign-functions=16 ",
-            4.0 => " -Wfatal-errors"
+            4.0 => ""
+                #. " -Wfatal-errors"
                 . " -Wmissing-field-initializers"
                 . " -Wmissing-include-dirs"
                 . " -Wvariadic-macros"
-                . " -Wno-discard-qual"
+                #. " -Wno-discard-qual"
                 . " -Wno-pointer-sign",
             4.1 => " -Wc++-compat",
             4.2 => "",
