@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: jit.pm 19972 2007-07-18 17:56:46Z jisom $
+# $Id: jit.pm 21164 2007-09-09 12:26:45Z bernhard $
 
 =head1 NAME
 
@@ -36,11 +36,17 @@ sub runstep {
     }
 
     my $verbose = $conf->options->get('verbose');
+    $verbose and print $/;
 
     my $jitbase  = 'src/jit';                      # base path for jit sources
     my $archname = $conf->data->get('archname');
     my ( $cpuarch, $osname ) = split( /-/, $archname );
 
+    if ( $verbose ) {
+        print "determining operating system and cpu architecture$/";
+        print "archname: <$archname>$/";
+    }
+
     if ( !defined $osname ) {
         ( $osname, $cpuarch ) = ( $cpuarch, q{} );
     }
@@ -59,8 +65,9 @@ sub runstep {
             $cpuarch = 'ppc';
         }
     }
+    # cpuarch and osname are reversed in archname on windows
     elsif ( $cpuarch =~ /MSWin32/ ) {
-        $cpuarch = 'i386';
+        $cpuarch = ( $osname =~ /x64/ ) ? 'amd64' : 'i386';
         $osname  = 'MSWin32';
     }
     elsif ( $osname =~ /cygwin/i || $cpuarch =~ /cygwin/i ) {
@@ -76,6 +83,8 @@ sub runstep {
     $cpuarch =~ s/i[456]86/i386/i;
     $cpuarch =~ s/x86_64/amd64/i;
 
+    warn "osname: $osname\ncpuarch: $cpuarch\n" if $verbose;
+
     $conf->data->set(
         archname => $archname,
         cpuarch  => $cpuarch,
@@ -89,20 +98,25 @@ sub runstep {
         -e "$jitbase/$cpuarch/core.jit" ? 'yes' : 'no', "\n" )
         if $verbose;
 
-    # RT#43145 disable all but i386, ppc
-    my %jit_is_working = (
-        i386 => 1,
-        ppc  => 1,
+    if ( -e "$jitbase/$cpuarch/core.jit" ) {
+        # Just because there is a "$jitbase/$cpuarch/core.jit" file,
+        # doesn't mean the JIT is working on that platform.
+        # So build JIT per default only on platforms where JIT in known
+        # to work. Building JIT on other platform most likely breaks the build.
+        # Developer can always call: Configure.pl --jitcapable
+        # See also RT#43145
+        my %jit_is_working = (
+            i386 => 1,
+            ppc  => 1,
+        );
+        if ( $jit_is_working{$cpuarch} ) {
+            $jitcapable = 1;
+        }
 
-        # all others are seriously b0rked
-    );
-
-    if ( -e "$jitbase/$cpuarch/core.jit" && $jit_is_working{$cpuarch} ) {
-        $jitcapable = 1;
-    }
-
-    if ( $cpuarch eq 'i386' and $osname eq 'darwin' ) {
-        $jitcapable = 0;
+        # Another exception
+        if ( $cpuarch eq 'i386' && $osname eq 'darwin' ) {
+            $jitcapable = 0;
+        }
     }
 
     if ( -e "$jitbase/$cpuarch/$jitarchname.s" ) {
@@ -117,6 +131,7 @@ sub runstep {
         $conf->data->set( asmfun_o => '' );
     }
 
+    # let developers override the default JIT capability
     $jitcapable = $conf->options->get('jitcapable')
         if defined $conf->options->get('jitcapable');
 
