@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: readline.pm 19060 2007-06-17 14:21:35Z paultcochrane $
+# $Id: readline.pm 20842 2007-08-25 17:56:03Z chromatic $
 
 =head1 NAME
 
@@ -44,9 +44,6 @@ sub runstep {
             $conf->data->add( ' ', libs => 'readline.lib' );
         }
     }
-    elsif ( $^O =~ /linux/i ) {
-        $conf->data->add( ' ', libs => '-lreadline -lncurses' );
-    }
     else {
         $conf->data->add( ' ', libs => '-lreadline' );
     }
