@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2003, The Perl Foundation.
-# $Id: /parrotcode/local/config/init/hints.pm 733 2006-12-17T23:24:17.491923Z chromatic  $
+# $Id: hints.pm 21127 2007-09-07 23:55:12Z jkeenan $
 
 =head1 NAME
 
@@ -40,21 +40,22 @@ sub runstep {
     eval "use $hints";
     die $@ if $@;
 
-    # call the runstep method if it exists.  Otherwise the step must have done
-    # it's work when it was loaded.
+    # Call the runstep method if it exists.
+    # Otherwise the step must have done its work when it was loaded.
     $hints->runstep( $conf, @_ ) if $hints->can('runstep');
     $hints_used++;
 
     $hints = "init::hints::local";
     print "$hints " if $verbose;
     eval "use $hints";
+
     unless ($@) {
         $hints->runstep( $conf, @_ ) if $hints->can('runstep');
         $hints_used++;
     }
 
-    if ( $hints_used == 0 ) {
-        print "(no hints) " if $verbose;
+    if ( $hints_used == 0 and $verbose ) {
+        print "(no hints) ";
     }
 
     print "]" if $verbose;
