@@ -1,5 +1,5 @@
 # Copyright (C) 2005-2007, The Perl Foundation.
-# $Id: linux.pm 20433 2007-08-02 18:57:48Z petdance $
+# $Id: linux.pm 21187 2007-09-11 17:35:57Z paultcochrane $
 
 package init::hints::linux;
 
@@ -8,14 +8,19 @@ use warnings;
 
 use Config;
 
+our $verbose;
+
 sub runstep {
     my ( $self, $conf ) = @_;
 
     my $libs        = $conf->option_or_data('libs');
-    my $cc_flags    = $conf->option_or_data('ccflags');
+    my $ccflags     = $conf->option_or_data('ccflags');
     my $cc          = $conf->option_or_data('cc');
     my $linkflags   = $conf->option_or_data('linkflags');
 
+    $verbose = $conf->options->get( 'verbose' );
+    print $/ if $verbose;
+
     # should find g++ in most cases
     my $link = $conf->data->get('link') || 'c++';
 
@@ -30,15 +35,44 @@ sub runstep {
         $link = $cc;
 
         # don't allow icc to pretend it's gcc
-        $cc_flags .= ' -no-gcc';
+        $ccflags .= ' -no-gcc';
 
         # suppress sprintf warnings that don't apply
-        $cc_flags .= ' -wd269';
+        $ccflags .= ' -wd269';
 
-        $cc_flags .= ' -Wall -Wcheck -w2';
+        # suppress remarks about floating point comparisons
+        $ccflags .= ' -wd1572';
+
+        # gcc is currently not looking for unused variables, so should icc
+        # for the time being (this will reduce the noise somewhat)
+        $ccflags .= ' -wd869';
+
+        # ignore "operands are evaluated in unspecified order" warning
+        $ccflags .= ' -wd981';
+
+        # ignore "external declaration in primary source file"
+        # (only done temporarily to reduce noise)
+        $ccflags .= ' -wd1419';
+
+        # ignore "function 'xxx' was declared but never referenced"
+        # (only done temporarily to reduce noise)
+        $ccflags .= ' -wd117';
+
+        # ignore "missing return statement at end of non-void function"
+        # warnings (only done temporarily to reduce noise)
+        $ccflags .= ' -wd1011';
+
+        # ignore warnings springing from problems with computed goto
+        # statements.  If someone can find out how to make icc play nicely
+        # in these situations, that would be good.
+        $ccflags .= ' -wd1296';
+
+        $ccflags .= ' -Wall -Wcheck -w2';
 
         $ld_share_flags = ' -shared -g -pipe -fexceptions -fPIC';
         $cc_shared .= ' -fPIC';
+
+        $verbose and print " ccflags: $ccflags\n";
     }
     elsif ( $cc =~ /suncc/ ) {
         $link = 'sunCC';
@@ -61,16 +95,16 @@ sub runstep {
         $linkflags .= ' -Wl,-E';
     }
 
-    if ( $cc_flags !~ /-D_GNU_SOURCE/ ) {
+    if ( $ccflags !~ /-D_GNU_SOURCE/ ) {
 
         # Request visibility of all POSIX symbols
         # _XOPEN_SOURCE=600 doesn't work with glibc 2.1.3
         # _XOPEN_SOURCE=500 gives 2 undefined warns (setenv, unsetenv) on 2.1.3
-        $cc_flags .= ' -D_GNU_SOURCE';
+        $ccflags .= ' -D_GNU_SOURCE';
     }
 
     $conf->data->set(
-        ccflags        => $cc_flags,
+        ccflags        => $ccflags,
         libs           => $libs,
         ld_share_flags => $ld_share_flags,
         ld_load_flags  => $ld_share_flags,
