@@ -1,5 +1,5 @@
 # Copyright (C) 2005, The Perl Foundation.
-# $Id: /parrotcode/local/config/init/hints/vms.pm 733 2006-12-17T23:24:17.491923Z chromatic  $
+# $Id: vms.pm 20912 2007-08-29 13:49:56Z bernhard $
 
 package init::hints::vms;
 
@@ -11,9 +11,9 @@ sub runstep {
 
     $conf->data->set(
         ccflags =>
-            qq{/Standard=Relaxed_ANSI/Prefix=All/Obj=.obj/NoList/NOANSI_ALIAS/include="./include"},
+            qq{/Prefix=All/Obj=.obj/NoList/include="./include"/nowarn},
         perl => "MCR $^X",
-        exe  => "exe"
+        exe  => ".exe"
     );
 
     {
