@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: install.pm 20204 2007-07-26 02:25:16Z jkeenan $
+# $Id: install.pm 21196 2007-09-12 00:46:53Z jkeenan $
 
 =head1 NAME
 
@@ -66,7 +66,9 @@ sub runstep {
     my ( $self, $conf ) = @_;
 
     my $prefix  = $conf->options->get('prefix')      || "/usr/local";
+    $prefix =~ s{/\z}{};
     my $ep = $conf->options->get('exec-prefix');
+    $ep =~ s{/\z}{} if defined $ep;
     my $eprefix = $ep ? $ep : $prefix;
 
     #  --bindir=DIR           user executables [EPREFIX/bin]
