@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2003, The Perl Foundation.
-# $Id: /parrotcode/local/config/inter/charset.pm 733 2006-12-17T23:24:17.491923Z chromatic  $
+# $Id: charset.pm 20919 2007-08-30 09:12:17Z bernhard $
 
 =head1 NAME
 
@@ -17,6 +17,8 @@ use strict;
 use warnings;
 use vars qw($description @args);
 
+use File::Basename qw/basename/;
+
 use base qw(Parrot::Configure::Step::Base);
 
 use Parrot::Configure::Step ':inter';
@@ -30,7 +32,7 @@ sub runstep {
 
     my @charset = (
         sort
-            map { m{\./src/charset/(.*)} } glob "./src/charset/*.c"
+            map { basename($_) } glob "./src/charset/*.c"
     );
 
     my $charset_list = $conf->options->get('charset')
