@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2003, The Perl Foundation.
-# $Id: /parrotcode/local/config/inter/encoding.pm 733 2006-12-17T23:24:17.491923Z chromatic  $
+# $Id: encoding.pm 20919 2007-08-30 09:12:17Z bernhard $
 
 =head1 NAME
 
@@ -19,6 +19,8 @@ use vars qw($description @args);
 
 use base qw(Parrot::Configure::Step::Base);
 
+use File::Basename qw/basename/;
+
 use Parrot::Configure::Step ':inter';
 
 $description = 'Determining what encoding files should be compiled in';
@@ -30,7 +32,7 @@ sub runstep {
 
     my @encoding = (
         sort
-            map { m{\./src/encodings/(.*)} } glob "./src/encodings/*.c"
+            map { basename($_) } glob "./src/encodings/*.c"
     );
 
     my $encoding_list = $conf->options->get('encoding')
