@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: progs.pm 18877 2007-06-08 14:15:40Z paultcochrane $
+# $Id: progs.pm 21212 2007-09-12 12:58:28Z paultcochrane $
 
 =head1 NAME
 
@@ -21,30 +21,22 @@ use vars qw($description @args);
 
 use base qw(Parrot::Configure::Step::Base);
 
-use Parrot::Configure::Step ':inter';
+use Parrot::Configure::Step ':inter', ':auto';
 
 $description = 'Determining what C compiler and linker to use';
 
 @args = qw(ask cc cxx link ld ccflags ccwarn linkflags ldflags libs debugging
     lex yacc maintainer);
 
+our $verbose;
+
 sub runstep {
     my ( $self, $conf ) = @_;
 
-    my ( $cc, $cxx, $link, $ld, $ccflags, $ccwarn, $linkflags, $ldflags, $libs, $lex, $yacc );
+    $verbose = $conf->options->get( 'verbose' );
+    print $/ if $verbose;
 
-    # Find a working version of a program:
-    # Try each alternative, until one works.
-    # If none work, then set to null command.
-    # RT#43173 need config support for a null command.
-    my $null          = 'echo';
-    my $first_working = sub {
-        foreach (@_) {
-            `$_ -h 2>&1`;
-            return $_ if not $?;
-        }
-        return $null;
-    };
+    my ( $cc, $cxx, $link, $ld, $ccflags, $ccwarn, $linkflags, $ldflags, $libs, $lex, $yacc );
 
     my $ask = $conf->options->get('ask');
     if ($ask) {
@@ -82,11 +74,12 @@ END
     $ccflags =~ s/-D((PERL|HAVE)_\w+\s*|USE_PERLIO)//g;
     $ccflags =~ s/-fno-strict-aliasing//g;
     $ccflags =~ s/-fnative-struct//g;
-    $ccflags = integrate( $ccflags, $conf->options->get('ccflags') );
     $ccflags = prompt( "What flags should your C compiler receive?", $ccflags )
         if $ask;
     $conf->data->set( ccflags => $ccflags );
 
+    $verbose and print " ccflags: $ccflags\n";
+
     $linkflags = $conf->data->get('linkflags');
     $linkflags =~ s/-libpath:\S+//g;    # RT#43174 No idea why.
     $linkflags = integrate( $linkflags, $conf->options->get('linkflags') );
@@ -115,8 +108,11 @@ END
     $debug = 'y' if $conf->options->get('debugging');
     $debug = prompt( "Do you want a debugging build of Parrot?", $debug )
         if $ask;
+    unless ($debug =~ /^[yn]$/i) {
+        return;
+    }
 
-    if ( !$debug || $debug =~ /n/i ) {
+    if ( $debug =~ /n/i ) {
         $conf->data->set(
             cc_debug   => '',
             link_debug => '',
@@ -128,9 +124,34 @@ END
     $ccwarn = integrate( $conf->data->get('ccwarn'), $conf->options->get('ccwarn') );
     $conf->data->set( ccwarn => $ccwarn );
 
+    test_compiler( $cc );
+
     return $self;
 }
 
+sub test_compiler
+{
+    my $cc = shift;
+
+    open( my $out_fh, '>', 'test.c' ) or die "Unable to open 'test.c': $@\n";
+    print {$out_fh} <<END_C;
+int main() {
+    return 0;
+}
+END_C
+    close $out_fh;
+
+    unless ( eval { cc_build(); 1 } ) {
+        warn "Compilation failed with '$cc'\n";
+        exit 1;
+    }
+
+    unless ( eval { cc_run(); 1 } ) {
+        warn $@ if $@;
+        exit 1;
+    }
+}
+
 1;
 
 # Local Variables:
