@@ -3,7 +3,7 @@
  *  Copyright (C) 2002, The Perl Foundation.
  *  License:  Artistic/GPL, see README and LICENSES for details
  *  SVN Info
- *     $Id: datatypes.h 19881 2007-07-14 22:05:00Z petdance $
+ *     $Id: datatypes.h 21143 2007-09-08 19:11:14Z bernhard $
  *  Overview:
  *     Parrot and native data types enums and type names.
  *
@@ -124,7 +124,7 @@ const struct _data_types data_types[] = {
 };
 #else
 extern const struct _data_types data_types[];
-#endif
+#endif /* INSIDE_GLOBAL_SETUP */
 
 /* HEADERIZER BEGIN: src/datatypes.c */
 
