@@ -1,7 +1,7 @@
 /* dod.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: dod.h 20663 2007-08-18 01:37:27Z chromatic $
+ *     $Id: dod.h 21245 2007-09-13 06:27:48Z chromatic $
  *  Overview:
  *     Handles dead object destruction of the various headers
  *  History:
@@ -15,10 +15,10 @@
 
 /* Macros for recursively blocking and unblocking DOD */
 #define Parrot_block_DOD(interp) \
-        do { \
+        { \
             (interp)->arena_base->DOD_block_level++; \
             Parrot_shared_DOD_block(interp); \
-        } while (0)
+        }
 
 #define Parrot_unblock_DOD(interp) \
         if ((interp)->arena_base->DOD_block_level) { \
@@ -51,43 +51,6 @@
 
 /* HEADERIZER BEGIN: src/gc/dod.c */
 
-PARROT_API
-void Parrot_do_dod_run( PARROT_INTERP, UINTVAL flags )
-        __attribute__nonnull__(1);
-
-PARROT_API
-void Parrot_dod_clear_live_bits( PARROT_INTERP )
-        __attribute__nonnull__(1);
-
-PARROT_API
-void Parrot_dod_ms_run( PARROT_INTERP, int flags )
-        __attribute__nonnull__(1);
-
-PARROT_API
-void Parrot_dod_ms_run_init( PARROT_INTERP )
-        __attribute__nonnull__(1);
-
-PARROT_API
-void Parrot_dod_profile_end( PARROT_INTERP, int what )
-        __attribute__nonnull__(1);
-
-PARROT_API
-void Parrot_dod_profile_start( PARROT_INTERP )
-        __attribute__nonnull__(1);
-
-PARROT_API
-void Parrot_dod_sweep( PARROT_INTERP, NOTNULL(Small_Object_Pool *pool) )
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
-
-PARROT_API
-int Parrot_dod_trace_children( PARROT_INTERP, size_t how_many )
-        __attribute__nonnull__(1);
-
-PARROT_API
-int Parrot_dod_trace_root( PARROT_INTERP, int trace_stack )
-        __attribute__nonnull__(1);
-
 PARROT_API
 void pobject_lives( PARROT_INTERP, NOTNULL(PObj *obj) )
         __attribute__nonnull__(1)
@@ -97,6 +60,70 @@ void clear_cow( PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int cleanup )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+void Parrot_do_dod_run( PARROT_INTERP, UINTVAL flags )
+        __attribute__nonnull__(1);
+
+void Parrot_dod_clear_live_bits( PARROT_INTERP )
+        __attribute__nonnull__(1);
+
+void Parrot_dod_free_buffer(
+     PARROT_INTERP,
+    NOTNULL(Small_Object_Pool *pool),
+    NOTNULL(PObj *b) )
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
+void Parrot_dod_free_buffer_malloc( PARROT_INTERP,
+    NOTNULL(Small_Object_Pool *pool),
+    NOTNULL(PObj *b) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
+void Parrot_dod_free_pmc( PARROT_INTERP,
+    NOTNULL(Small_Object_Pool *pool),
+    NOTNULL(PObj *p) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
+void Parrot_dod_free_sysmem( PARROT_INTERP,
+    NOTNULL(Small_Object_Pool *pool),
+    NOTNULL(PObj *b) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
+void Parrot_dod_ms_run( PARROT_INTERP, int flags )
+        __attribute__nonnull__(1);
+
+void Parrot_dod_ms_run_init( PARROT_INTERP )
+        __attribute__nonnull__(1);
+
+void Parrot_dod_profile_end( PARROT_INTERP, int what )
+        __attribute__nonnull__(1);
+
+void Parrot_dod_profile_start( PARROT_INTERP )
+        __attribute__nonnull__(1);
+
+void Parrot_dod_sweep( PARROT_INTERP, NOTNULL(Small_Object_Pool *pool) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
+int Parrot_dod_trace_children( PARROT_INTERP, size_t how_many )
+        __attribute__nonnull__(1);
+
+void Parrot_dod_trace_pmc_data( PARROT_INTERP, NOTNULL(PMC * const p) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
+int Parrot_dod_trace_root( PARROT_INTERP, int trace_stack )
+        __attribute__nonnull__(1);
+
+void Parrot_free_pmc_ext( PARROT_INTERP, NOTNULL(PMC *p) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
 void trace_mem_block( PARROT_INTERP, size_t lo_var_ptr, size_t hi_var_ptr )
         __attribute__nonnull__(1);
 
@@ -124,11 +151,9 @@ extern int CONSERVATIVE_POINTER_CHASING;
 /* GC subsystem init functions */
 /* HEADERIZER BEGIN: src/gc/gc_gms.c */
 
-PARROT_API
 void Parrot_gc_gms_init( PARROT_INTERP )
         __attribute__nonnull__(1);
 
-PARROT_API
 void parrot_gc_gms_pobject_lives( PARROT_INTERP, NOTNULL(PObj *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -159,13 +184,11 @@ void parrot_gc_gms_wb_key( PARROT_INTERP,
 
 /* HEADERIZER BEGIN: src/gc/gc_ims.c */
 
-PARROT_API
 void Parrot_dod_ims_wb( PARROT_INTERP, NOTNULL(PMC *agg), NOTNULL(PMC *_new) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
-PARROT_API
 void Parrot_gc_ims_init( PARROT_INTERP )
         __attribute__nonnull__(1);
 
