@@ -1,7 +1,7 @@
 /* encoding.h
  *  Copyright (C) 2004-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: encoding.h 19914 2007-07-16 21:06:48Z petdance $
+ *     $Id: encoding.h 20342 2007-07-30 05:57:58Z petdance $
  *  Overview:
  *     This is the header for the generic encoding functions
  *  Data Structure and Algorithms:
@@ -15,20 +15,20 @@
 
 #include "parrot/parrot.h"
 
-typedef STRING * (*encoding_to_encoding_t)(Interp*, STRING *src, STRING *dest);
-typedef UINTVAL (*encoding_get_codepoint_t)(Interp*, const STRING *src, UINTVAL offset);
-typedef void (*encoding_set_codepoint_t)(Interp*, STRING *src, UINTVAL offset, UINTVAL codepoint);
-typedef UINTVAL (*encoding_get_byte_t)(Interp*, const STRING *src, UINTVAL offset);
-typedef void (*encoding_set_byte_t)(Interp*, const STRING *src, UINTVAL offset, UINTVAL count);
-typedef STRING *(*encoding_get_codepoints_t)(Interp*, STRING *src, UINTVAL offset, UINTVAL count);
-typedef STRING *(*encoding_get_bytes_t)(Interp*, STRING *src, UINTVAL offset, UINTVAL count);
-typedef STRING *(*encoding_get_codepoints_inplace_t)(Interp*, STRING *src, UINTVAL offset, UINTVAL count, STRING *dest_string);
-typedef STRING *(*encoding_get_bytes_inplace_t)(Interp*, STRING *src, UINTVAL offset, UINTVAL count, STRING *dest_string);
-typedef void (*encoding_set_codepoints_t)(Interp*, STRING *src, UINTVAL offset, UINTVAL count, STRING *new_bytes);
-typedef void (*encoding_set_bytes_t)(Interp*, STRING *src, UINTVAL offset, UINTVAL count, STRING *new_bytes);
-typedef void (*encoding_become_encoding_t)(Interp*, STRING *src);
-typedef UINTVAL (*encoding_codepoints_t)(Interp*, STRING *src);
-typedef UINTVAL (*encoding_bytes_t)(Interp*, STRING *src);
+typedef STRING * (*encoding_to_encoding_t)(PARROT_INTERP, NOTNULL(STRING *src), NULLOK(STRING *dest));
+typedef UINTVAL (*encoding_get_codepoint_t)(PARROT_INTERP, const STRING *src, UINTVAL offset);
+typedef void (*encoding_set_codepoint_t)(PARROT_INTERP, STRING *src, UINTVAL offset, UINTVAL codepoint);
+typedef UINTVAL (*encoding_get_byte_t)(PARROT_INTERP, const STRING *src, UINTVAL offset);
+typedef void (*encoding_set_byte_t)(PARROT_INTERP, const STRING *src, UINTVAL offset, UINTVAL count);
+typedef STRING *(*encoding_get_codepoints_t)(PARROT_INTERP, STRING *src, UINTVAL offset, UINTVAL count);
+typedef STRING *(*encoding_get_bytes_t)(PARROT_INTERP, STRING *src, UINTVAL offset, UINTVAL count);
+typedef STRING *(*encoding_get_codepoints_inplace_t)(PARROT_INTERP, STRING *src, UINTVAL offset, UINTVAL count, STRING *dest_string);
+typedef STRING *(*encoding_get_bytes_inplace_t)(PARROT_INTERP, STRING *src, UINTVAL offset, UINTVAL count, STRING *dest_string);
+typedef void (*encoding_set_codepoints_t)(PARROT_INTERP, STRING *src, UINTVAL offset, UINTVAL count, STRING *new_bytes);
+typedef void (*encoding_set_bytes_t)(PARROT_INTERP, STRING *src, UINTVAL offset, UINTVAL count, STRING *new_bytes);
+typedef void (*encoding_become_encoding_t)(PARROT_INTERP, STRING *src);
+typedef UINTVAL (*encoding_codepoints_t)(PARROT_INTERP, STRING *src);
+typedef UINTVAL (*encoding_bytes_t)(PARROT_INTERP, STRING *src);
 
 /* iterator support */
 
@@ -38,23 +38,23 @@ typedef void (*encoding_iter_init_t)(PARROT_INTERP, const STRING *src,
         struct string_iterator_t *);
 
 struct _encoding {
-    const char *name;
+    NOTNULL(const char *name);
     UINTVAL max_bytes_per_codepoint;
-    encoding_to_encoding_t to_encoding;
-    encoding_get_codepoint_t get_codepoint;
-    encoding_set_codepoint_t  set_codepoint;
-    encoding_get_byte_t  get_byte;
-    encoding_set_byte_t  set_byte;
-    encoding_get_codepoints_t  get_codepoints;
-    encoding_get_codepoints_inplace_t  get_codepoints_inplace;
-    encoding_get_bytes_t  get_bytes;
-    encoding_get_bytes_inplace_t  get_bytes_inplace;
-    encoding_set_codepoints_t  set_codepoints;
-    encoding_set_bytes_t  set_bytes;
-    encoding_become_encoding_t  become_encoding;
-    encoding_codepoints_t  codepoints;
-    encoding_bytes_t  bytes;
-    encoding_iter_init_t     iter_init;
+    encoding_to_encoding_t              to_encoding;
+    encoding_get_codepoint_t            get_codepoint;
+    encoding_set_codepoint_t            set_codepoint;
+    encoding_get_byte_t                 get_byte;
+    encoding_set_byte_t                 set_byte;
+    encoding_get_codepoints_t           get_codepoints;
+    encoding_get_codepoints_inplace_t   get_codepoints_inplace;
+    encoding_get_bytes_t                get_bytes;
+    encoding_get_bytes_inplace_t        get_bytes_inplace;
+    encoding_set_codepoints_t           set_codepoints;
+    encoding_set_bytes_t                set_bytes;
+    encoding_become_encoding_t          become_encoding;
+    encoding_codepoints_t               codepoints;
+    encoding_bytes_t                    bytes;
+    encoding_iter_init_t                iter_init;
 };
 
 typedef struct _encoding ENCODING;
@@ -71,12 +71,12 @@ PARROT_API extern ENCODING *Parrot_default_encoding_ptr;
 #define PARROT_FIXED_8_ENCODING Parrot_fixed_8_encoding_ptr
 #define PARROT_DEFAULT_FOR_UNICODE_ENCODING NULL
 
-typedef INTVAL (*encoding_converter_t)(Interp *, ENCODING *lhs, ENCODING *rhs);
+typedef INTVAL (*encoding_converter_t)(PARROT_INTERP, ENCODING *lhs, ENCODING *rhs);
 
 /* HEADERIZER BEGIN: src/encoding.c */
 
 PARROT_API
-ENCODING * Parrot_default_encoding( SHIM_INTERP );
+const ENCODING * Parrot_default_encoding( SHIM_INTERP );
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
@@ -98,14 +98,14 @@ INTVAL Parrot_encoding_number( PARROT_INTERP, NOTNULL(STRING *encodingname) )
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
-INTVAL Parrot_encoding_number_of_str( PARROT_INTERP, NOTNULL(STRING *src) )
-        __attribute__nonnull__(1)
+INTVAL Parrot_encoding_number_of_str( SHIM_INTERP,
+    NOTNULL(const STRING *src) )
         __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
-ENCODING * Parrot_find_encoding( SHIM_INTERP,
+const ENCODING * Parrot_find_encoding( SHIM_INTERP,
     NOTNULL(const char *encodingname) )
         __attribute__nonnull__(2);
 
@@ -118,19 +118,20 @@ encoding_converter_t Parrot_find_encoding_converter( PARROT_INTERP,
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
-ENCODING* Parrot_get_encoding( PARROT_INTERP, INTVAL number_of_encoding )
+const ENCODING* Parrot_get_encoding( PARROT_INTERP,
+    INTVAL number_of_encoding )
         __attribute__nonnull__(1);
 
 PARROT_API
 PARROT_DOES_NOT_RETURN
-ENCODING * Parrot_load_encoding( PARROT_INTERP,
+const ENCODING * Parrot_load_encoding( PARROT_INTERP,
     NOTNULL(const char *encodingname) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
 INTVAL Parrot_make_default_encoding( SHIM_INTERP,
-    const char *encodingname,
+    SHIM(const char *encodingname),
     NOTNULL(ENCODING *encoding) )
         __attribute__nonnull__(3);
 
@@ -151,35 +152,35 @@ void parrot_init_encodings_2( void );
 /* HEADERIZER END: src/encoding.c */
 
 #define ENCODING_MAX_BYTES_PER_CODEPOINT(i, src) \
-    ((const ENCODING *)src->encoding)->max_bytes_per_codepoint
+    ((src)->encoding)->max_bytes_per_codepoint
 #define ENCODING_GET_CODEPOINT(i, src, offset) \
-    ((const ENCODING *)src->encoding)->get_codepoint(i, src, offset)
+    ((src)->encoding)->get_codepoint(i, src, offset)
 #define ENCODING_SET_CODEPOINT(i, src, offset, codepoint) \
-    ((const ENCODING *)src->encoding)->set_codepoint(i, src, offset, codepoint)
+    ((src)->encoding)->set_codepoint(i, src, offset, codepoint)
 #define ENCODING_GET_BYTE(i, src, offset) \
-    ((const ENCODING *)src->encoding)->get_byte(i, src, offset)
+    ((src)->encoding)->get_byte(i, src, offset)
 #define ENCODING_SET_BYTE(i, src, offset, value) \
-    ((const ENCODING *)src->encoding)->set_byte(i, src, offset, value)
+    ((src)->encoding)->set_byte(i, src, offset, value)
 #define ENCODING_GET_CODEPOINTS(i, src, offset, count) \
-    ((const ENCODING *)src->encoding)->get_codepoints(i, src, offset, count)
+    ((src)->encoding)->get_codepoints(i, src, offset, count)
 #define ENCODING_GET_CODEPOINTS_INPLACE(i, src, offset, count, dest) \
-    ((const ENCODING *)src->encoding)->get_codepoints_inplace(i, src, offset, count, dest)
+    ((src)->encoding)->get_codepoints_inplace(i, src, offset, count, dest)
 #define ENCODING_GET_BYTES(i, src, offset, count) \
-    ((const ENCODING *)src->encoding)->get_bytes(i, src, offset, count)
+    ((src)->encoding)->get_bytes(i, src, offset, count)
 #define ENCODING_GET_BYTES_INPLACE(i, src, offset, count, dest) \
-    ((const ENCODING *)src->encoding)->get_bytes_inplace(i, src, offset, count, dest)
+    ((src)->encoding)->get_bytes_inplace(i, src, offset, count, dest)
 #define ENCODING_SET_CODEPOINTS(i, src, offset, count, newdata) \
-    ((const ENCODING *)src->encoding)->set_codepoints(i, src, offset, count, newdata)
+    ((src)->encoding)->set_codepoints(i, src, offset, count, newdata)
 #define ENCODING_SET_BYTES(i, src, offset, count, newdata) \
-    ((const ENCODING *)src->encoding)->set_bytes(i, src, offset, count, newdata)
+    ((src)->encoding)->set_bytes(i, src, offset, count, newdata)
 #define ENCODING_BECOME_ENCODING(i, src) \
-    ((const ENCODING *)src->encoding)->become_encoding(i, src)
+    ((src)->encoding)->become_encoding(i, src)
 #define ENCODING_CODEPOINTS(i, src) \
-    ((const ENCODING *)src->encoding)->codepoints(i, src)
+    ((src)->encoding)->codepoints(i, src)
 #define ENCODING_BYTES(i, src) \
-    ((const ENCODING *)src->encoding)->bytes(i, src)
+    ((src)->encoding)->bytes(i, src)
 #define ENCODING_ITER_INIT(i, src, iter) \
-    ((const ENCODING *)src->encoding)->iter_init(i, src, iter)
+    ((src)->encoding)->iter_init(i, src, iter)
 
 #endif /* PARROT_ENCODING_H_GUARD */
 
