@@ -1,7 +1,7 @@
 /* exceptions.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: exceptions.h 19881 2007-07-14 22:05:00Z petdance $
+ *     $Id: exceptions.h 20378 2007-07-31 08:09:05Z paultcochrane $
  *  Overview:
  *     define the internal interpreter exceptions
  *  Data Structure and Algorithms:
@@ -176,6 +176,15 @@ PARROT_API
 void new_internal_exception( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_API
+PARROT_DOES_NOT_RETURN
+void Parrot_confess(
+    NOTNULL(const char *cond),
+    NOTNULL(const char *file),
+    unsigned int line )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
 PARROT_API
 void Parrot_pop_mark( PARROT_INTERP, INTVAL mark )
         __attribute__nonnull__(1);
@@ -219,7 +228,7 @@ opcode_t * rethrow_exception( PARROT_INTERP, NOTNULL(PMC *exception) )
 
 PARROT_API
 PARROT_CAN_RETURN_NULL
-opcode_t * throw_exception( PARROT_INTERP, PMC *exception, void *dest )
+opcode_t * throw_exception( PARROT_INTERP, PMC *exception, SHIM(void *dest) )
         __attribute__nonnull__(1);
 
 void destroy_exception_list( PARROT_INTERP )
@@ -230,11 +239,12 @@ void do_panic(
     NULLOK_INTERP,
     NULLOK(const char *message),
     NULLOK(const char *file),
-    int line );
+    unsigned int line );
 
 void Parrot_init_exceptions( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+void Parrot_print_backtrace( void );
 void really_destroy_exception_list( NULLOK(Parrot_exception *e) );
 PARROT_DOES_NOT_RETURN
 void rethrow_c_exception( PARROT_INTERP )
@@ -244,6 +254,13 @@ void rethrow_c_exception( PARROT_INTERP )
 
 #define PANIC(interp, message) do_panic(interp, message, __FILE__, __LINE__)
 
+#ifdef NDEBUG
+#  define PARROT_ASSERT(x) ((void)0)
+#else
+#  define PARROT_ASSERT(x) (((x) ? (void)0 : (void)Parrot_confess(#x, __FILE__, __LINE__)))
+#endif
+
+
 
 #endif /* PARROT_EXCEPTIONS_H_GUARD */
 
