@@ -1,7 +1,7 @@
 /* exit.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: exit.h 19881 2007-07-14 22:05:00Z petdance $
+ *     $Id: exit.h 20351 2007-07-30 15:32:47Z petdance $
  *  Overview:
  *
  *  Data Structure and Algorithms:
@@ -16,7 +16,7 @@
 
 #include "parrot/compiler.h"    /* compiler capabilities */
 
-typedef void (*exit_handler_f)(Interp *, int , void *);
+typedef void (*exit_handler_f)(PARROT_INTERP, int , void *);
 
 typedef struct _handler_node_t {
     exit_handler_f function;
@@ -32,9 +32,9 @@ void Parrot_exit( PARROT_INTERP, int status )
         __attribute__nonnull__(1);
 
 PARROT_API
-int Parrot_on_exit( PARROT_INTERP,
+void Parrot_on_exit( PARROT_INTERP,
     NOTNULL(exit_handler_f function),
-    void *arg )
+    NULLOK(void *arg) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
