@@ -1,7 +1,7 @@
 /* hash.h
  *  Copyright (C) 2001-2003, The Perl Foundation.
  *  SVN Info
- *     $Id: hash.h 20126 2007-07-23 18:42:27Z petdance $
+ *     $Id: hash.h 20994 2007-09-02 04:40:38Z chromatic $
  *  Overview:
  *     Hashtable implementation
  *  Data Structure and Algorithms:
@@ -189,6 +189,14 @@ PARROT_API
 void parrot_new_pointer_hash( SHIM_INTERP, NOTNULL(Hash **hptr) )
         __attribute__nonnull__(2);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_PURE_FUNCTION
+int int_compare( SHIM_INTERP, NULLOK(const void *a), NULLOK(const void *b) );
+
+PARROT_WARN_UNUSED_RESULT
+PARROT_PURE_FUNCTION
+size_t key_hash_int( SHIM_INTERP, NULLOK(void *value), size_t seed );
+
 void parrot_chash_destroy( PARROT_INTERP, NOTNULL(Hash *hash) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
