@@ -1,7 +1,7 @@
 /* headers.h
  *  Copyright (C) 2001-2003, The Perl Foundation.
  *  SVN Info
- *     $Id: headers.h 19817 2007-07-12 15:48:43Z petdance $
+ *     $Id: headers.h 20085 2007-07-22 04:37:48Z petdance $
  *  Overview:
  *     Header management functions. Handles getting of various headers,
  *     and pool creation
@@ -42,7 +42,7 @@ typedef enum {
     POOL_ALL    = 0x07
 } pool_iter_enum;
 
-typedef int (*pool_iter_fn)(Interp *, struct Small_Object_Pool *, int, void*);
+typedef int (*pool_iter_fn)(PARROT_INTERP, struct Small_Object_Pool *, int, void*);
 
 
 /* HEADERIZER BEGIN: src/headers.c */
