@@ -1,7 +1,7 @@
 /* hll.h
  *  Copyright (C) 2005-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: hll.h 20661 2007-08-17 22:39:03Z chromatic $
+ *     $Id: hll.h 21270 2007-09-14 01:15:24Z chromatic $
  *  Overview:
  *     This is the API header for the HLL subsystem
  *  Data Structure and Algorithms:
@@ -57,10 +57,14 @@ void Parrot_regenerate_HLL_namespaces( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 PARROT_API
-INTVAL Parrot_register_HLL( PARROT_INTERP,
-    NULLOK(STRING *hll_name),
-    NULLOK(STRING *hll_lib) )
-        __attribute__nonnull__(1);
+INTVAL Parrot_register_HLL( PARROT_INTERP, NOTNULL(STRING *hll_name) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
+PARROT_API
+INTVAL Parrot_register_HLL_lib( PARROT_INTERP, NOTNULL(STRING *hll_lib) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
 void Parrot_register_HLL_type( PARROT_INTERP,
@@ -69,12 +73,16 @@ void Parrot_register_HLL_type( PARROT_INTERP,
     INTVAL hll_type )
         __attribute__nonnull__(1);
 
+void Parrot_init_HLL( PARROT_INTERP )
+        __attribute__nonnull__(1);
+
 /* HEADERIZER END: src/hll.c */
 
 typedef enum HLL_enum_t {
     e_HLL_name,
     e_HLL_lib,
     e_HLL_typemap,
+    e_HLL_id,
     e_HLL_MAX
 } HLL_enum_t;
 
