@@ -1,7 +1,7 @@
 /* interpreter.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: interpreter.h 20687 2007-08-18 20:16:33Z chromatic $
+ *     $Id: interpreter.h 21144 2007-09-08 19:31:26Z bernhard $
  *  Overview:
  *     The interpreter API handles running the operations
  */
@@ -358,7 +358,7 @@ struct parrot_interp_t {
     Interp * parent_interpreter;
 
     /* per interpreter global vars */
-    INTVAL world_inited;                      /* Parrot_init is done */
+    INTVAL world_inited;                      /* world_init_once() is done */
 
     PMC *iglobals;                            /* SArray of PMCs, containing: */
     /* 0:   PMC *Parrot_base_classname_hash; hash containing name->base_type */
@@ -484,9 +484,6 @@ VAR_SCOPE native_func_t run_native;
 typedef PMC *(*Parrot_compiler_func_t)(Parrot_Interp interp,
                                        const char * program );
 
-/* embed.c */
-PARROT_API void Parrot_init(PARROT_INTERP);
-
 /* HEADERIZER BEGIN: src/inter_create.c */
 
 PARROT_API
