@@ -1,7 +1,7 @@
 /* io.h
  *  Copyright (C) 2001-2003, The Perl Foundation.
  *  SVN Info
- *     $Id: io.h 19882 2007-07-14 22:33:30Z petdance $
+ *     $Id: io.h 20342 2007-07-30 05:57:58Z petdance $
  *  Overview:
  *      Parrot IO subsystem
  *  Data Structure and Algorithms:
@@ -146,6 +146,8 @@ extern INTVAL pio_errno;
 /* HEADERIZER BEGIN: src/io/io.c */
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC * new_io_pmc( PARROT_INTERP, NULLOK(ParrotIO *io) )
         __attribute__nonnull__(1);
 
@@ -155,12 +157,14 @@ void Parrot_IOData_mark( PARROT_INTERP, NOTNULL(ParrotIOData *piodata) )
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PMC * PIO_accept( PARROT_INTERP, NOTNULL(PMC *pmc) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
-INTVAL PIO_base_init( SHIM_INTERP, ParrotIOLayer *l );
+INTVAL PIO_base_init( SHIM_INTERP, SHIM(ParrotIOLayer *l) );
 
 PARROT_API
 INTVAL PIO_bind( PARROT_INTERP, NOTNULL(PMC *pmc), NOTNULL(STRING *address) )
@@ -195,6 +199,7 @@ INTVAL PIO_eprintf( NULLOK(PARROT_INTERP), NOTNULL(const char *s), ... )
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC * PIO_fdopen( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
     PIOHANDLE fd,
@@ -249,11 +254,16 @@ INTVAL PIO_listen( PARROT_INTERP, NOTNULL(PMC *pmc), INTVAL backlog )
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
-ParrotIO * PIO_new( PARROT_INTERP, INTVAL iotype, INTVAL flags, INTVAL mode )
+PARROT_CANNOT_RETURN_NULL
+ParrotIO * PIO_new( PARROT_INTERP,
+    NULLOK(INTVAL iotype),
+    INTVAL flags,
+    INTVAL mode )
         __attribute__nonnull__(1);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC * PIO_open( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer),
     NOTNULL(const char *spath),
@@ -314,6 +324,7 @@ INTVAL PIO_read( PARROT_INTERP,
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * PIO_reads( PARROT_INTERP, NOTNULL(PMC *pmc), size_t len )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -351,18 +362,26 @@ INTVAL PIO_setlinebuf( PARROT_INTERP, NOTNULL(PMC *pmc) )
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC * PIO_socket( PARROT_INTERP, INTVAL fam, INTVAL type, INTVAL proto )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC * PIO_STDERR( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC * PIO_STDIN( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC * PIO_STDOUT( PARROT_INTERP )
         __attribute__nonnull__(1);
 
@@ -383,6 +402,7 @@ INTVAL PIO_write( PARROT_INTERP,
         __attribute__nonnull__(3);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * PIO_make_io_string( PARROT_INTERP,
     NOTNULL(STRING **buf),
     size_t len )
@@ -391,8 +411,9 @@ STRING * PIO_make_io_string( PARROT_INTERP,
 
 PIOOFF_T PIO_make_offset( INTVAL offset );
 PIOOFF_T PIO_make_offset32( INTVAL hi, INTVAL lo );
-PIOOFF_T PIO_make_offset_pmc( PARROT_INTERP, PMC *pmc )
-        __attribute__nonnull__(1);
+PIOOFF_T PIO_make_offset_pmc( PARROT_INTERP, NOTNULL(PMC *pmc) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 /* HEADERIZER END: src/io/io.c */
 
@@ -403,17 +424,24 @@ PARROT_API
 void PIO_base_delete_layer( NULLOK(ParrotIOLayer *layer) );
 
 PARROT_API
+PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 ParrotIOLayer * PIO_base_new_layer( NULLOK(ParrotIOLayer *proto) );
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 ParrotIOLayer * PIO_copy_stack( NULLOK(ParrotIOLayer *stack) );
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 PARROT_API
 ParrotIOLayer * PIO_get_layer( SHIM_INTERP, NOTNULL(const char *name) )
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_IGNORABLE_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIOLayer * PIO_pop_layer( PARROT_INTERP, NULLOK(PMC *pmc) )
         __attribute__nonnull__(1);
 
@@ -423,24 +451,29 @@ INTVAL PIO_push_layer( PARROT_INTERP,
     NULLOK(ParrotIOLayer *layer) )
         __attribute__nonnull__(1);
 
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * PIO_pop_layer_str( PARROT_INTERP, NOTNULL(PMC *pmc) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-void PIO_push_layer_str( PARROT_INTERP, PMC *pmc, STRING *ls )
-        __attribute__nonnull__(1);
+void PIO_push_layer_str( PARROT_INTERP,
+    NOTNULL(PMC *pmc),
+    NULLOK(STRING *ls) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 /* HEADERIZER END: src/io/io_layers.c */
 
 /* Put platform specific macros here if you must */
 #ifdef PIO_OS_WIN32
-extern STRING          *PIO_sockaddr_in(Interp *, unsigned short, STRING *);
+extern STRING          *PIO_sockaddr_in(PARROT_INTERP, unsigned short, STRING *);
 extern INTVAL           PIO_win32_getblksize(PIOHANDLE fd);
 #  define PIO_getblksize(x)   PIO_win32_getblksize(x)
 #endif
 
 #ifdef PIO_OS_UNIX
-extern STRING          *PIO_sockaddr_in(Interp *, unsigned short, STRING *);
+extern STRING          *PIO_sockaddr_in(PARROT_INTERP, unsigned short, STRING *);
 extern INTVAL           PIO_unix_getblksize(PIOHANDLE fd);
 #  define PIO_getblksize(x)   PIO_unix_getblksize(x)
 #endif
