@@ -1,7 +1,7 @@
 /* key.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: key.h 20042 2007-07-20 20:07:50Z petdance $
+ *     $Id: key.h 20994 2007-09-02 04:40:38Z chromatic $
  *  Overview:
  *     This is the API header for the PMC subsystem
  *  Data Structure and Algorithms:
@@ -36,6 +36,9 @@ typedef enum {
 
 } KEY_flags;
 
+#define KEY_IS_HASH_ITERATOR(k) \
+    ((PObj_get_FLAGS((k)) & KEY_type_FLAGS) == KEY_hash_iterator_FLAGS)
+
 /* HEADERIZER BEGIN: src/key.c */
 
 PARROT_API
