@@ -1,7 +1,7 @@
 /* library.h
  *  Copyright (C) 2004, The Perl Foundation.
  *  SVN Info
- *     $Id: library.h 19882 2007-07-14 22:33:30Z petdance $
+ *     $Id: library.h 20138 2007-07-24 05:19:59Z petdance $
  *  Overview:
  *      Contains accessor functions for the _parrotlib bytecode interface
  *  Data Structure and Algorithms:
@@ -36,11 +36,14 @@ typedef enum {
 /* HEADERIZER BEGIN: src/library.c */
 
 PARROT_API
+PARROT_MALLOC
+PARROT_CAN_RETURN_NULL
 char* Parrot_get_runtime_prefix( PARROT_INTERP, NULLOK(STRING **prefix_str) )
         __attribute__nonnull__(1);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 char* Parrot_locate_runtime_file( PARROT_INTERP,
     NOTNULL(const char *file_name),
     enum_runtime_ft type )
@@ -49,6 +52,7 @@ char* Parrot_locate_runtime_file( PARROT_INTERP,
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 STRING* Parrot_locate_runtime_file_str( PARROT_INTERP,
     NOTNULL(STRING *file),
     enum_runtime_ft type )
@@ -58,11 +62,14 @@ STRING* Parrot_locate_runtime_file_str( PARROT_INTERP,
 void parrot_init_library_paths( PARROT_INTERP )
         __attribute__nonnull__(1);
 
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * parrot_split_path_ext( PARROT_INTERP,
-    STRING *in,
+    NOTNULL(STRING *in),
     NOTNULL(STRING **wo_ext),
     NOTNULL(STRING **ext) )
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
