@@ -3,7 +3,7 @@
  *  Copyright (C) 2002, The Perl Foundation.
  *  License:  Artistic/GPL, see README and LICENSES for details
  *  SVN Info
- *     $Id: list.h 19885 2007-07-15 04:02:08Z petdance $
+ *     $Id: list.h 20037 2007-07-20 18:23:03Z petdance $
  *  Overview:
  *     list aka array routines for Parrot
  *     s. list.c for more
@@ -87,7 +87,7 @@ PARROT_API
 void list_assign( PARROT_INTERP,
     NOTNULL(List *list),
     INTVAL idx,
-    void *item,
+    NULLOK(void *item),
     int type )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
@@ -135,6 +135,7 @@ void list_mark( PARROT_INTERP, NOTNULL(List *list) )
 
 PARROT_API
 PARROT_MALLOC
+PARROT_CANNOT_RETURN_NULL
 List * list_new( PARROT_INTERP, PARROT_DATA_TYPE type )
         __attribute__nonnull__(1);
 
@@ -195,14 +196,18 @@ void list_splice( PARROT_INTERP,
         __attribute__nonnull__(2);
 
 PARROT_API
-void list_unshift( PARROT_INTERP, NOTNULL(List *list), void *item, int type )
+void list_unshift( PARROT_INTERP,
+    NOTNULL(List *list),
+    NULLOK(void *item),
+    int type )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
-void list_visit( PARROT_INTERP, NOTNULL(List *list), void *pinfo )
+void list_visit( PARROT_INTERP, NOTNULL(List *list), NOTNULL(void *pinfo) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 /* HEADERIZER END: src/list.c */
 
