@@ -1,7 +1,7 @@
 /* parrot.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: parrot.h 19790 2007-07-11 20:09:10Z petdance $
+ *     $Id: parrot.h 20258 2007-07-27 08:00:07Z petdance $
  *  Overview:
  *     General header file includes for the parrot interpreter
  *  Data Structure and Algorithms:
@@ -241,8 +241,8 @@ typedef void (*funcptr_t)(void);
 #endif /* O_BINARY */
 
 /* Hide our struct copying behind macros */
-#define STRUCT_COPY(d,s)    assert(d);assert(s);*(d)=*(s);
-#define STRUCT_COPY_N(d,s,n) assert(d);assert(s);assert(sizeof(*d)==sizeof(*s));memcpy((d),(s),sizeof(*d)*(n));
+#define STRUCT_COPY(d,s)    (PARROT_ASSERT(d),PARROT_ASSERT(s),*(d)=*(s))
+#define STRUCT_COPY_N(d,s,n) (PARROT_ASSERT(d),PARROT_ASSERT(s),PARROT_ASSERT(sizeof(*d)==sizeof(*s)),memcpy((d),(s),sizeof(*d)*(n)))
 
 #include "parrot/settings.h"
 #include "parrot/enums.h"
@@ -297,6 +297,7 @@ typedef void (*funcptr_t)(void);
 #include "parrot/slice.h"
 #include "parrot/hll.h"
 #include "parrot/stm/backend.h"
+#include "parrot/pbcversion.h"
 
 #endif /* PARROT_PARROT_H_GUARD */
 
