@@ -1,7 +1,7 @@
 /* pic.h
  *  Copyright (C) 2005, The Perl Foundation.
  *  SVN Info
- *     $Id: pic.h 19905 2007-07-16 04:44:05Z petdance $
+ *     $Id: pic.h 20085 2007-07-22 04:37:48Z petdance $
  *  Overview:
  *     This is the api header for the pic subsystem
  *  Data Structure and Algorithms:
@@ -65,14 +65,19 @@ typedef struct Parrot_pic_store_t {
     size_t n_mics;                      /* range check, debugging mainly */
 } Parrot_PIC_store;
 
-typedef int (*arg_pass_f)(Interp *, PMC *sig,
+typedef int (*arg_pass_f)(PARROT_INTERP, PMC *sig,
             char *src_base, void **src_pc, char *dest_base, void **dest_pc);
 
 /* more or less private interfaces */
 
 /* HEADERIZER BEGIN: src/pic.c */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 Parrot_MIC* parrot_PIC_alloc_mic( const PARROT_INTERP, size_t n );
+
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 Parrot_PIC* parrot_PIC_alloc_pic( PARROT_INTERP )
         __attribute__nonnull__(1);
 
@@ -105,6 +110,8 @@ void parrot_pic_find_infix_v_pp( PARROT_INTERP,
 PARROT_CONST_FUNCTION
 int parrot_PIC_op_is_cached( int op_code );
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 void * parrot_pic_opcode( PARROT_INTERP, INTVAL op )
         __attribute__nonnull__(1);
 
@@ -120,13 +127,17 @@ void parrot_PIC_prederef( PARROT_INTERP,
 
 /* HEADERIZER BEGIN: src/pic_jit.c */
 
+PARROT_WARN_UNUSED_RESULT
 int parrot_pic_is_safe_to_jit( PARROT_INTERP,
     NOTNULL(PMC *sub),
-    PMC *sig_args,
-    PMC *sig_results,
-    int *flags )
+    NOTNULL(PMC *sig_args),
+    NOTNULL(PMC *sig_results),
+    NOTNULL(int *flags) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4)
+        __attribute__nonnull__(5);
 
 funcptr_t parrot_pic_JIT_sub( PARROT_INTERP, NOTNULL(PMC *sub), int flags )
         __attribute__nonnull__(1)
