@@ -1,7 +1,7 @@
 /* string_funcs.h
  *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: string_funcs.h 19914 2007-07-16 21:06:48Z petdance $
+ *     $Id: string_funcs.h 20353 2007-07-30 15:50:06Z petdance $
  *  Overview:
  *     This is the api header for the string subsystem
  *  Data Structure and Algorithms:
@@ -33,6 +33,7 @@ STRING * const_string( PARROT_INTERP, NOTNULL(const char *buffer) )
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * int_to_str( PARROT_INTERP,
     NOTNULL(char *tc),
     HUGEINTVAL num,
@@ -41,22 +42,23 @@ STRING * int_to_str( PARROT_INTERP,
         __attribute__nonnull__(2);
 
 PARROT_API
-PARROT_CAN_RETURN_NULL
+PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
-STRING * Parrot_make_COW_reference( PARROT_INTERP, NULLOK(STRING *s) )
-        __attribute__nonnull__(1);
+STRING * Parrot_make_COW_reference( PARROT_INTERP, NOTNULL(STRING *s) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_CANNOT_RETURN_NULL
-STRING* Parrot_reuse_COW_reference( SHIM_INTERP,
+STRING * Parrot_reuse_COW_reference( SHIM_INTERP,
     NOTNULL(STRING *s),
     NOTNULL(STRING *d) )
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
+PARROT_CANNOT_RETURN_NULL
 const char * Parrot_string_cstring( SHIM_INTERP, NOTNULL(const STRING *str) )
         __attribute__nonnull__(2);
 
@@ -112,40 +114,47 @@ void Parrot_unmake_COW( PARROT_INTERP, NOTNULL(STRING *s) )
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * string_append( PARROT_INTERP, NULLOK(STRING *a), NULLOK(STRING *b) )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING * string_bitwise_and( PARROT_INTERP,
     NULLOK(STRING *s1),
     NULLOK(STRING *s2),
-    NULLOK(STRING **dest) )
+    ARGOUT_NULLOK(STRING **dest) )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING * string_bitwise_not( PARROT_INTERP,
     NULLOK(STRING *s),
-    NULLOK(STRING **dest) )
+    ARGOUT_NULLOK(STRING **dest) )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING * string_bitwise_or( PARROT_INTERP,
     NULLOK(STRING *s1),
     NULLOK(STRING *s2),
-    NULLOK(STRING **dest) )
+    ARGOUT_NULLOK(STRING **dest) )
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING * string_bitwise_xor( PARROT_INTERP,
     NULLOK(STRING *s1),
     NULLOK(STRING *s2),
-    NULLOK(STRING **dest) )
+    ARGOUT_NULLOK(STRING **dest) )
         __attribute__nonnull__(1);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
-INTVAL string_bool( PARROT_INTERP, NULLOK(const STRING *s) )
-        __attribute__nonnull__(1);
+INTVAL string_bool( PARROT_INTERP, NOTNULL(const STRING *s) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
@@ -184,12 +193,13 @@ STRING * string_compose( PARROT_INTERP, NULLOK(STRING *src) )
         __attribute__nonnull__(1);
 
 PARROT_API
-PARROT_MAY_IGNORE_RESULT
+PARROT_IGNORABLE_RESULT
 INTVAL string_compute_strlen( PARROT_INTERP, NOTNULL(STRING *s) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING * string_concat( PARROT_INTERP,
     NULLOK(STRING *a),
     NULLOK(STRING *b),
@@ -211,7 +221,7 @@ void string_deinit( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PARROT_MALLOC
 STRING * string_downcase( PARROT_INTERP, NOTNULL(const STRING *s) )
         __attribute__nonnull__(1)
@@ -230,6 +240,7 @@ INTVAL string_equal( PARROT_INTERP,
         __attribute__nonnull__(1);
 
 PARROT_API
+PARROT_CAN_RETURN_NULL
 STRING * string_escape_string( PARROT_INTERP, NULLOK(const STRING *src) )
         __attribute__nonnull__(1);
 
@@ -251,11 +262,13 @@ STRING * string_from_cstring( PARROT_INTERP,
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * string_from_int( PARROT_INTERP, INTVAL i )
         __attribute__nonnull__(1);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * string_from_num( PARROT_INTERP, FLOATVAL f )
         __attribute__nonnull__(1);
 
@@ -273,8 +286,9 @@ size_t string_hash( PARROT_INTERP, NULLOK(STRING *s), size_t seed )
 PARROT_API
 PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
-STRING * string_increment( PARROT_INTERP, NULLOK(const STRING *s) )
-        __attribute__nonnull__(1);
+STRING * string_increment( PARROT_INTERP, NOTNULL(const STRING *s) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
@@ -294,9 +308,9 @@ STRING* string_join( PARROT_INTERP, NULLOK(STRING *j), NOTNULL(PMC *ar) )
         __attribute__nonnull__(3);
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
 PARROT_PURE_FUNCTION
-UINTVAL string_length( SHIM_INTERP, NULLOK(const STRING *s) );
+UINTVAL string_length( SHIM_INTERP, NOTNULL(const STRING *s) )
+        __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
@@ -314,14 +328,15 @@ PARROT_CANNOT_RETURN_NULL
 STRING * string_make_direct( PARROT_INTERP,
     NULLOK(const char *buffer),
     UINTVAL len,
-    NOTNULL(ENCODING *encoding),
-    NOTNULL(CHARSET *charset),
+    NOTNULL(const ENCODING *encoding),
+    NOTNULL(const CHARSET *charset),
     UINTVAL flags )
         __attribute__nonnull__(1)
         __attribute__nonnull__(4)
         __attribute__nonnull__(5);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING * string_make_empty( PARROT_INTERP,
     parrot_string_representation_t representation,
     UINTVAL capacity )
@@ -335,8 +350,9 @@ INTVAL string_max_bytes( SHIM_INTERP,
         __attribute__nonnull__(2);
 
 PARROT_API
-STRING* string_nprintf( PARROT_INTERP,
-    STRING *dest,
+PARROT_CANNOT_RETURN_NULL
+STRING * string_nprintf( PARROT_INTERP,
+    NULLOK(STRING *dest),
     INTVAL bytelen,
     NOTNULL(const char *format),
     ... )
@@ -345,8 +361,9 @@ STRING* string_nprintf( PARROT_INTERP,
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
-INTVAL string_ord( PARROT_INTERP, NULLOK(const STRING *s), INTVAL idx )
-        __attribute__nonnull__(1);
+INTVAL string_ord( PARROT_INTERP, NOTNULL(const STRING *s), INTVAL idx )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
 void string_pin( PARROT_INTERP, NOTNULL(STRING *s) )
@@ -360,26 +377,28 @@ const char* string_primary_encoding_for_representation( PARROT_INTERP,
         __attribute__nonnull__(1);
 
 PARROT_API
-STRING* string_printf( PARROT_INTERP, NOTNULL(const char *format), ... )
+PARROT_CANNOT_RETURN_NULL
+STRING * string_printf( PARROT_INTERP, NOTNULL(const char *format), ... )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
-CHARSET * string_rep_compatible( SHIM_INTERP,
+const CHARSET * string_rep_compatible( SHIM_INTERP,
     NOTNULL(const STRING *a),
     NOTNULL(const STRING *b),
-    NOTNULL(ENCODING **e) )
+    ARGOUT(const ENCODING **e) )
         __attribute__nonnull__(2)
         __attribute__nonnull__(3)
         __attribute__nonnull__(4);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING * string_repeat( PARROT_INTERP,
     NOTNULL(const STRING *s),
     UINTVAL num,
-    NULLOK(STRING **d) )
+    ARGOUT_NULLOK(STRING **d) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
@@ -390,7 +409,7 @@ STRING * string_replace( PARROT_INTERP,
     INTVAL offset,
     INTVAL length,
     NOTNULL(STRING *rep),
-    NULLOK(STRING **d) )
+    ARGOUT_NULLOK(STRING **d) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         __attribute__nonnull__(5);
@@ -405,6 +424,7 @@ STRING * string_set( PARROT_INTERP,
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PMC* string_split( PARROT_INTERP,
     NOTNULL(STRING *delim),
     NOTNULL(STRING *str) )
@@ -423,17 +443,19 @@ INTVAL string_str_index( PARROT_INTERP,
         __attribute__nonnull__(3);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 STRING * string_substr( PARROT_INTERP,
     NOTNULL(STRING *src),
     INTVAL offset,
     INTVAL length,
-    NULLOK(STRING **d),
+    ARGOUT_NULLOK(STRING **d),
     int replace_dest )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PARROT_MALLOC
 STRING * string_titlecase( PARROT_INTERP, NOTNULL(const STRING *s) )
         __attribute__nonnull__(1)
@@ -445,21 +467,24 @@ void string_titlecase_inplace( PARROT_INTERP, NOTNULL(STRING *s) )
         __attribute__nonnull__(2);
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
 PARROT_MALLOC
-PARROT_CAN_RETURN_NULL
-char * string_to_cstring( SHIM_INTERP, NULLOK(const STRING *s) );
+PARROT_CANNOT_RETURN_NULL
+char * string_to_cstring( SHIM_INTERP, NOTNULL(const STRING *s) )
+        __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
-INTVAL string_to_int( SHIM_INTERP, NULLOK(const STRING *s) );
+INTVAL string_to_int( SHIM_INTERP, NOTNULL(const STRING *s) )
+        __attribute__nonnull__(2);
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
-FLOATVAL string_to_num( PARROT_INTERP, NULLOK(const STRING *s) )
-        __attribute__nonnull__(1);
+FLOATVAL string_to_num( PARROT_INTERP, NOTNULL(const STRING *s) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_API
+PARROT_CANNOT_RETURN_NULL
 STRING * string_unescape_cstring( PARROT_INTERP,
     NOTNULL(const char *cstring),
     char delimiter,
@@ -473,7 +498,7 @@ void string_unpin( PARROT_INTERP, NOTNULL(STRING *s) )
         __attribute__nonnull__(2);
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 PARROT_MALLOC
 STRING * string_upcase( PARROT_INTERP, NOTNULL(const STRING *s) )
         __attribute__nonnull__(1)
@@ -486,6 +511,7 @@ void string_upcase_inplace( PARROT_INTERP, NOTNULL(STRING *s) )
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING* uint_to_str( PARROT_INTERP,
     NOTNULL(char *tc),
     UHUGEINTVAL num,
