@@ -1,7 +1,7 @@
 /* string_funcs.h
  *  Copyright (C) 2001-2003, The Perl Foundation.
  *  SVN Info
- *     $Id: string_primitives.h 19833 2007-07-13 22:02:25Z petdance $
+ *     $Id: string_primitives.h 20015 2007-07-19 16:11:36Z petdance $
  *  Overview:
  *     This is the api header for the string subsystem
  *  Data Structure and Algorithms:
@@ -18,27 +18,21 @@
 /* HEADERIZER BEGIN: src/string_primitives.c */
 
 PARROT_API
+PARROT_CONST_FUNCTION
 UINTVAL Parrot_char_digit_value( SHIM_INTERP, UINTVAL character );
 
 PARROT_API
-void string_fill_from_buffer( PARROT_INTERP,
-    NOTNULL(const void *buffer),
-    UINTVAL len,
-    const char *encoding_name,
-    NULLOK(STRING *s) )
+void string_set_data_directory( PARROT_INTERP, NOTNULL(const char *dir) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-PARROT_API
-void string_set_data_directory( PARROT_INTERP, const char *dir )
-        __attribute__nonnull__(1);
-
 PARROT_API
 Parrot_UInt4 string_unescape_one( PARROT_INTERP,
     NOTNULL(UINTVAL *offset),
-    STRING *string )
+    NOTNULL(STRING *string) )
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 /* HEADERIZER END: src/string_primitives.c */
 
