@@ -1,7 +1,7 @@
 /* thr_pthread.h
  *  Copyright (C) 2001-2003, The Perl Foundation.
  *  SVN Info
- *     $Id: thr_pthread.h 18945 2007-06-12 14:08:35Z fperrad $
+ *     $Id: thr_pthread.h 20663 2007-08-18 01:37:27Z chromatic $
  *  Overview:
  *     POSIS pthread interface
  *  Data Structure and Algorithms:
@@ -19,8 +19,8 @@
 #  include <pthread.h>
 
 #  define PARROT_SYNC_PRIMITIVES_DEFINED
-#  define LOCK(m) pthread_mutex_lock(&m)
-#  define UNLOCK(m) pthread_mutex_unlock(&m)
+#  define LOCK(m) pthread_mutex_lock((pthread_mutex_t*)&m)
+#  define UNLOCK(m) pthread_mutex_unlock((pthread_mutex_t*)&m)
 #  define COND_WAIT(c,m) pthread_cond_wait(&c, &m)
 #  define COND_TIMED_WAIT(c,m,t) pthread_cond_timedwait(&c, &m, t)
 #  define COND_SIGNAL(c) pthread_cond_signal(&c)
@@ -38,11 +38,11 @@
 #  define THREAD_CREATE_DETACHED(t, func, arg) do { \
         pthread_attr_t      attr;   \
         int rc = pthread_attr_init(&attr);      \
-        assert(rc == 0);    \
+        PARROT_ASSERT(rc == 0);    \
         rc = pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);   \
-        assert(rc == 0);    \
+        PARROT_ASSERT(rc == 0);    \
         rc = pthread_create(&t, &attr, func, arg); \
-        assert(rc == 0);    \
+        PARROT_ASSERT(rc == 0);    \
         pthread_attr_destroy(&attr);        \
    } while (0)
 
