@@ -1,7 +1,7 @@
 /* thread.h
  *  Copyright (C) 2001-2003, The Perl Foundation.
  *  SVN Info
- *     $Id: thr_windows.h 18945 2007-06-12 14:08:35Z fperrad $
+ *     $Id: thr_windows.h 20404 2007-08-01 16:57:11Z particle $
  *  Overview:
  *     This is the api header for the windows thread primitives
  *  Data Structure and Algorithms:
@@ -65,7 +65,7 @@ typedef HANDLE Parrot_thread;
        { \
          ++(c).m_lWaiters; \
          UNLOCK(m); \
-         diff = (t->tv_sec - sec)*1000L + (t->tv_nsec - nsec)/1000000L; \
+         diff = (DWORD)((t->tv_sec - sec)*1000L + (t->tv_nsec - nsec)/1000000L); \
          WaitForSingleObject((c).m_hSema, diff); \
          LOCK(m); \
          --(c).m_lWaiters; \
