@@ -1,7 +1,7 @@
 /* thread.h
- *  Copyright (C) 2001-2003, The Perl Foundation.
+ *  Copyright (C) 2001-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: thread.h 18945 2007-06-12 14:08:35Z fperrad $
+ *     $Id: thread.h 20574 2007-08-09 04:39:16Z petdance $
  *  Overview:
  *     This is the api header for the thread primitives
  *  Data Structure and Algorithms:
@@ -161,41 +161,63 @@ typedef struct _Sync {
     Parrot_mutex pmc_lock;              /* for wr access to PMCs content */
 } Sync;
 
-/*
- * thread.c interface functions
- */
-int  pt_thread_run(Parrot_Interp, PMC* dest_interp, PMC* sub, PMC *arg);
-int  pt_thread_run_1(Parrot_Interp, PMC* dest_interp, PMC* sub, PMC *arg);
-int  pt_thread_run_2(Parrot_Interp, PMC* dest_interp, PMC* sub, PMC *arg);
-int  pt_thread_run_3(Parrot_Interp, PMC* dest_interp, PMC* sub, PMC *arg);
+/* HEADERIZER BEGIN: src/thread.c */
 
-void pt_thread_prepare_for_run(Parrot_Interp d, Parrot_Interp s);
-void pt_clone_code(Parrot_Interp d, Parrot_Interp s);
-void pt_clone_globals(Parrot_Interp d, Parrot_Interp s);
-void pt_add_to_interpreters(Parrot_Interp first, Parrot_Interp new_interp);
-void pt_thread_yield(void);
-PMC* pt_thread_join(Parrot_Interp, UINTVAL);
-void pt_thread_detach(UINTVAL);
-void pt_thread_kill(UINTVAL);
-void pt_join_threads(Parrot_Interp);
-void pt_suspend_self_for_gc(Parrot_Interp);
+PARROT_API
+void Parrot_shared_DOD_block( PARROT_INTERP )
+        __attribute__nonnull__(1);
 
-PMC *pt_shared_fixup(Parrot_Interp, PMC*);
+PARROT_API
+void Parrot_shared_DOD_unblock( PARROT_INTERP )
+        __attribute__nonnull__(1);
 
-PMC *pt_transfer_sub(Parrot_Interp dest, Parrot_Interp source, PMC *sub);
+void pt_add_to_interpreters( PARROT_INTERP, Parrot_Interp new_interp )
+        __attribute__nonnull__(1);
 
-void pt_thread_wait_with(Parrot_Interp, Parrot_mutex *);
+void pt_clone_code( Parrot_Interp d, Parrot_Interp s );
+void pt_clone_globals( Parrot_Interp d, Parrot_Interp s );
+void pt_DOD_mark_root_finished( PARROT_INTERP )
+        __attribute__nonnull__(1);
 
-/*
- * DOD interface
- */
-void pt_DOD_start_mark(Parrot_Interp);
-void pt_DOD_mark_root_finished(Parrot_Interp);
-void pt_DOD_stop_mark(Parrot_Interp);
+void pt_DOD_start_mark( PARROT_INTERP )
+        __attribute__nonnull__(1);
 
+void pt_DOD_stop_mark( PARROT_INTERP )
+        __attribute__nonnull__(1);
 
-PARROT_API void Parrot_shared_DOD_block(Parrot_Interp);
-PARROT_API void Parrot_shared_DOD_unblock(Parrot_Interp);
+void pt_join_threads( PARROT_INTERP )
+        __attribute__nonnull__(1);
+
+PMC * pt_shared_fixup( PARROT_INTERP, PMC *pmc )
+        __attribute__nonnull__(1);
+
+void pt_suspend_self_for_gc( PARROT_INTERP )
+        __attribute__nonnull__(1);
+
+void pt_thread_detach( UINTVAL tid );
+PMC* pt_thread_join( NOTNULL(Parrot_Interp parent), UINTVAL tid )
+        __attribute__nonnull__(1);
+
+void pt_thread_kill( UINTVAL tid );
+void pt_thread_prepare_for_run( Parrot_Interp d, Parrot_Interp s );
+int pt_thread_run( PARROT_INTERP, PMC* dest_interp, PMC* sub, PMC *arg )
+        __attribute__nonnull__(1);
+
+int pt_thread_run_1( PARROT_INTERP, PMC* dest_interp, PMC* sub, PMC *arg )
+        __attribute__nonnull__(1);
+
+int pt_thread_run_2( PARROT_INTERP, PMC* dest_interp, PMC* sub, PMC *arg )
+        __attribute__nonnull__(1);
+
+int pt_thread_run_3( PARROT_INTERP, PMC* dest_interp, PMC* sub, PMC *arg )
+        __attribute__nonnull__(1);
+
+void pt_thread_wait_with( PARROT_INTERP, Parrot_mutex *mutex )
+        __attribute__nonnull__(1);
+
+void pt_thread_yield( void );
+PMC * pt_transfer_sub( Parrot_Interp d, Parrot_Interp s, NULLOK(PMC *sub) );
+/* HEADERIZER END: src/thread.c */
 
 #endif /* PARROT_THREAD_H_GUARD */
 
