@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: BuildUtil.pm 20610 2007-08-11 18:14:09Z jkeenan $
+# $Id: BuildUtil.pm 21150 2007-09-09 02:03:14Z jkeenan $
 
 =head1 NAME
 
@@ -7,9 +7,9 @@ lib/Parrot/BuildUtil.pm - Utilities for building Parrot
 
 =head1 DESCRIPTION
 
-For now, this package contains only one subroutine:  C<parrot_version()>.
-This subroutine is not exported and so must be requested with a fully
-qualified path.
+This package holds two subroutines:  C<parrot_version()> and
+<slurp_file>.  Neither subroutine is exported and so each must be
+requested with a fully qualified path.
 
 =cut
 
@@ -17,7 +17,7 @@ package Parrot::BuildUtil;
 use strict;
 use warnings;
 
-=head2 SUBROUTINES
+=head1 SUBROUTINES
 
 =over 4
 
@@ -32,8 +32,6 @@ and returns it in a context-appropriate manner.
     @parrot_version = Parrot::BuildUtil::parrot_version();
     # @parrot_version is (0, 4, 11)
 
-=back
-
 =cut
 
 # cache for repeated calls
@@ -69,8 +67,29 @@ sub parrot_version {
     return wantarray ? @parrot_version : $parrot_version;
 }
 
+=item C<slurp_file($filename)>
+
+Slurps up the filename and returns the content as one string.  While
+doing so, it converts all DOS-style line endings to newlines.
+
+=cut
+
+sub slurp_file {
+    my ($file_name) = @_;
+
+    open( my $SLURP, '<', $file_name ) or die "open '$file_name': $!";
+    local $/ = undef;
+    my $file = <$SLURP> . '';
+    $file =~ s/\cM\cJ/\n/g;
+    close $SLURP;
+
+    return $file;
+}
+
 1;
 
+=back
+
 =head1 AUTHOR
 
 Gregor N. Purdy.  Revised by James E Keenan.
