@@ -1,5 +1,5 @@
-# Copyright (C) 2001-2006, The Perl Foundation.
-# $Id: Configure.pm 20603 2007-08-11 02:11:28Z jkeenan $
+# Copyright (C) 2001-2007, The Perl Foundation.
+# $Id: Configure.pm 21188 2007-09-11 17:41:49Z bernhard $
 
 =head1 NAME
 
@@ -251,15 +251,15 @@ sub _run_this_step {
 
     my $conftrace = [];
     my $sto = q{.configure_trace.sto};
-    if ($conf->options->get(q{configure_trace}) and (-e $sto)) {
-        $conftrace = retrieve($sto);
+    {
+        local $Storable::Eval = 1;
+        if ($conf->options->get(q{configure_trace}) and (-e $sto)) {
+            $conftrace = retrieve($sto);
+        }
     }
-    my $step = $step_name->new;
+    my $step = $step_name->new();
 
-    # RT#43675 This works. but is probably not a good design.
-    # Using $step->description() would be nicer
-    my $description = $step->description();
-    $description = "" unless defined $description;
+    my $description = $step->description() || q{};
 
     # set per step verbosity
     if ( defined $args->{verbose_step} ) {
@@ -336,7 +336,10 @@ sub _run_this_step {
             data    => $conf->{data},
         };
         push @{$conftrace}, $evolved_data;
-        nstore($conftrace, $sto);
+        {
+            local $Storable::Deparse = 1;
+            nstore($conftrace, $sto);
+        }
     }
 }
 
