@@ -1,8 +1,10 @@
 # Copyright (C) 2001-2006, The Perl Foundation.
-# $Id: Test.pm 18803 2007-06-04 14:03:54Z paultcochrane $
+# $Id: Test.pm 21099 2007-09-06 01:26:50Z jkeenan $
 package Parrot::Configure::Options::Test;
 use strict;
 use warnings;
+use Config;      # to find the correct $Config{scriptdir}/prove
+use File::Spec;  # to construct the path to the correct 'prove'
 
 our @preconfiguration_tests = qw(
     t/configure/*.t
@@ -38,8 +40,10 @@ sub run_configure_tests {
     my $self = shift;
     if ($self->{run_configure_tests}) {
         print "As you requested, we'll start with some tests of the configuration tools.\n\n";
-        system(qq{prove @preconfiguration_tests})
-            and die "Unable to execute configuration tests";
+        # Find the 'prove' command associated with *this* version of perl.
+        my $prove = File::Spec->catfile($Config{'scriptdir'} , 'prove');
+        system(qq{$prove @preconfiguration_tests})
+            and die "Pre-configuration tests did not complete successfully; Configure.pl will not continue.";
         print <<"TEST";
 
 I just ran some tests to demonstrate that
@@ -55,8 +59,9 @@ sub run_build_tests {
     if ($self->{run_build_tests}) {
         print "\n\n";
         print "As you requested, I will now run some tests of the build tools.\n\n";
-        system(qq{prove @postconfiguration_tests})
-            and die "Unable to execute post-configuration and build tools tests";
+        my $prove = File::Spec->catfile($Config{'scriptdir'} , 'prove');
+        system(qq{$prove @postconfiguration_tests})
+            and die "Post-configuration and build tools tests did not complete successfully; running 'make' might be dubious.";
     }
     return 1;
 }
@@ -78,9 +83,7 @@ In F<Configure.pl>:
 
     $args = process_options( {
         argv            => [ @ARGV ],
-        script          => $0,
-        parrot_version  => $parrot_version,
-        svnid           => '$Id: Test.pm 18803 2007-06-04 14:03:54Z paultcochrane $',
+        mode            => q{configure},
     } );
 
     $opttest = Parrot::Configure::Options::Test->new($args);
