@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: Step.pm 20748 2007-08-21 01:56:29Z jkeenan $
+# $Id: Step.pm 21281 2007-09-14 17:18:51Z paultcochrane $
 
 =head1 NAME
 
@@ -61,7 +61,7 @@ sub integrate {
     my ( $orig, $new ) = @_;
 
     # Rather than sprinkling "if defined(...)", everywhere,
-    # various inter::* steps (coded in config/inter/*.pm) permit simply 
+    # various inter::* steps (coded in config/inter/*.pm) permit simply
     # passing in potentially undefined strings.
     # In these instances, we simply pass back the original string without
     # generating a warning.
@@ -152,7 +152,7 @@ its contents have changed.
 
 =cut
 
-sub move_if_diff {
+sub move_if_diff {  ## no critic Subroutines::RequireFinalReturn
     my ( $from, $to, $ignore_pattern ) = @_;
     copy_if_diff( $from, $to, $ignore_pattern );
     unlink $from;
@@ -372,7 +372,7 @@ sub genfile {
             goto GMAKES if $any_gmake;
         }
 
-        # interoplate @foo@ values
+        # interpolate @foo@ values
         $line =~ s{ \@ (\w+) \@ }{
             if(defined(my $val=$conf->data->get($1))) {
                 #use Data::Dumper;warn Dumper("val for $1 is ",$val);
@@ -513,12 +513,18 @@ sub cc_build {
         );
 
     my $compile_command = _build_compile_command( $cc, $ccflags, $cc_args );
-    _run_command( $compile_command, 'test.cco', 'test.cco', $verbose )
+    my $compile_result = _run_command( $compile_command, 'test.cco', 'test.cco', $verbose )
         and confess "C compiler failed (see test.cco)";
+    if ( $compile_result ) {
+        return $compile_result;
+    }
 
-    _run_command( "$link $linkflags test$o $link_args ${cc_exe_out}test$exe $libs",
+    my $link_result = _run_command( "$link $linkflags test$o $link_args ${cc_exe_out}test$exe $libs",
         'test.ldo', 'test.ldo', $verbose )
         and confess "Linker failed (see test.ldo)";
+    if ( $link_result ) {
+        return $link_result;
+    }
 }
 
 =item C<cc_run()>
@@ -532,13 +538,15 @@ sub cc_run {
     my $exe     = $conf->data->get('exe');
     my $slash   = $conf->data->get('slash');
     my $verbose = $conf->options->get('verbose');
+    my $test_exe = ".${slash}test${exe}";
 
+    my $run_error;
     if ( defined( $_[0] ) && length( $_[0] ) ) {
         local $" = ' ';
-        _run_command( ".${slash}test${exe} @_", './test.out', undef, $verbose );
+        $run_error = _run_command( "$test_exe @_", './test.out', undef, $verbose );
     }
     else {
-        _run_command( ".${slash}test${exe}", './test.out', undef, $verbose );
+        $run_error = _run_command( "$test_exe", './test.out', undef, $verbose );
     }
 
     my $output = _slurp('./test.out');
@@ -577,7 +585,7 @@ Cleans up all files in the root folder that match the glob F<test.*>.
 
 =cut
 
-sub cc_clean {
+sub cc_clean { ## no critic Subroutines::RequireFinalReturn
     unlink map "test$_", qw( .c .cco .ldo .out), $conf->data->get(qw( o exe ));
 }
 
