@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2006, The Perl Foundation.
-# $Id: List.pm 18157 2007-04-12 00:32:35Z jkeenan $
+# $Id: List.pm 20427 2007-08-02 14:29:54Z petdance $
 package Parrot::Configure::Step::List;
 use strict;
 use warnings;
@@ -19,7 +19,9 @@ my @steps = qw(
     inter::lex
     inter::yacc
     auto::gcc
+    auto::backtrace
     auto::msvc
+    auto::attributes
     init::optimize
     inter::shlibs
     inter::libparrot
