@@ -1,5 +1,5 @@
 # Copyright (C) 2007, The Perl Foundation.
-# $Id: Test.pm 20408 2007-08-01 21:27:39Z paultcochrane $
+# $Id: Test.pm 20891 2007-08-29 02:07:22Z jkeenan $
 package Parrot::Configure::Test;
 use strict;
 use warnings;
@@ -11,7 +11,7 @@ our ( @ISA, @EXPORT_OK );
 use Carp;
 *ok = *Test::More::ok;
 *isa_ok = *Test::More::isa_ok;
-use lib qw( . lib ../lib ../../lib );
+use lib qw( lib );
 use Parrot::Configure;
 
 my $stepnum = -1;
@@ -60,7 +60,7 @@ Set-up for C<test_step_thru_runstep()>:
     argv            => [ ],
     script          => $0,
     parrot_version  => $parrot_version,
-    svnid           => '$Id: Test.pm 20408 2007-08-01 21:27:39Z paultcochrane $',
+    svnid           => '$Id: Test.pm 20891 2007-08-29 02:07:22Z jkeenan $',
 } );
 
     $conf = Parrot::Configure->new;
