@@ -1,5 +1,5 @@
 # Copyright (C) 2002, The Perl Foundation.
-# $Id: CGP.pm 18744 2007-06-02 01:10:38Z chromatic $
+# $Id: CGP.pm 19849 2007-07-14 08:13:57Z chromatic $
 
 =head1 NAME
 
@@ -118,7 +118,7 @@ code.
 sub goto_pop {
     my ($self) = @_;
 
-    return "goto **(cur_opcode = opcode_to_prederef(interp,
+    return "goto *(opcode_t *)(cur_opcode = opcode_to_prederef(interp,
         (opcode_t*)pop_dest(interp)))";
 }
 
