@@ -1,5 +1,5 @@
 # Copyright (C) 2007, The Perl Foundation.
-# $Id: Utils.pm 19849 2007-07-14 08:13:57Z chromatic $
+# $Id: Utils.pm 20126 2007-07-23 18:42:27Z petdance $
 package Parrot::Ops2c::Utils;
 use strict;
 use warnings;
@@ -507,9 +507,9 @@ sub _iterate_over_ops {
     $prev_src = '';
     foreach my $op ( $self->{ops}->ops ) {
         my $func_name = $op->func_name( $self->{trans} );
-        my $arg_types = "$self->{opsarraytype} *, Interp *";
+        my $arg_types = "$self->{opsarraytype} *, PARROT_INTERP";
         my $prototype = "$self->{sym_export} $self->{opsarraytype} * $func_name ($arg_types)";
-        my $args      = "$self->{opsarraytype} *cur_opcode, Interp *interp";
+        my $args      = "$self->{opsarraytype} *cur_opcode, PARROT_INTERP";
         my $definition;
         my $comment = '';
         my $one_op  = "";
