@@ -1,5 +1,5 @@
 # Copyright (C) 2006-2007, The Perl Foundation.
-# $Id: Harness.pm 19144 2007-06-19 19:40:09Z particle $
+# $Id: Harness.pm 19530 2007-07-01 22:35:56Z paultcochrane $
 
 =head1 NAME
 
@@ -95,7 +95,12 @@ If called with no args, run the suite.
     elsif ( @{ $options{arguments} } ) {
 
         # Someone specified tests for me to run.
-        return grep { -f $_ } map { glob $_ } @{ $options{arguments} };
+        my @files = ();
+        foreach my $arg (@{ $options{arguments} }) {
+            -f $arg && push @files, glob $arg;
+            -d $arg && push @files, glob( File::Spec->catfile( $arg, '*.t' ) );
+        }
+        return @files;
     }
     else {
 
@@ -122,7 +127,7 @@ sub import {
     local $Test::Harness::Switches = '';
 
     no warnings 'redefine';
-    local *Test::Harness::Straps::_INC2PERL5LIB = sub { @INC }
+    local *Test::Harness::Straps::_INC2PERL5LIB = sub { @INC }  ## no critic Variables::ProhibitConditionalDeclarations
         if $options{compiler};
 
     runtests(@files);
