@@ -1,5 +1,5 @@
 # Copyright (C) 2001-2007, The Perl Foundation.
-# $Id: Vtable.pm 19845 2007-07-14 04:45:42Z petdance $
+# $Id: Vtable.pm 20628 2007-08-15 02:00:35Z jkeenan $
 
 =head1 NAME
 
@@ -37,21 +37,20 @@ our @EXPORT = qw(parse_vtable vtbl_defs vtbl_struct vtbl_macros vtbl_embed);
 
 sub make_re {
     my $re = shift;
-
-    my $comp_re = eval "qr/$re/";
-
-    if   ($@) { return "(?:$re)"; }
-    else      { return $comp_re; }
+    my $comp_re = qr/$re/;
+    return (defined $comp_re)
+        ? $comp_re
+        : "(?:$re)";
 }
 
 my $ident_re   = make_re('[A-Za-z_][A-Za-z0-9_]*');
 my $type_re    = make_re( '(?:(?:struct\s+)|(?:union\s+))?' . $ident_re . '\**' );
 my $param_re   = make_re( $type_re . '\s+' . $ident_re );
 my $arglist_re = make_re( '(?:' . $param_re . '(?:\s*,\s*' . $param_re . ')*)?' );
-my $method_re =
+my $method_re  =
     make_re( '^\s*(' . $type_re . ')\s+(' . $ident_re . ')\s*\((' . $arglist_re . ')\)\s*$' );
-my $attrs_re = make_re('(?::(\w+)\s*)*');
-my $attr_re  = make_re(':(\w+)\s*');
+my $attrs_re   = make_re('(?::(\w+)\s*)*');
+my $attr_re    = make_re(':(\w+)\s*');
 
 sub parse_attrs {
     my $attrs = shift;
@@ -75,11 +74,10 @@ defaults to F<vtable.tbl>.  If it is not an MMD method, C<MMD_type> is -1.
 
 sub parse_vtable {
 
-    my $file   = defined $_[0] ? shift() : 'vtable.tbl';
-    my $vtable = [];
-    my $mmd    = [];
-    my $fh     = FileHandle->new( $file, O_RDONLY )
-        or die "Can't open $file for reading: $!\n";
+    my $file    = defined $_[0] ? shift() : 'vtable.tbl';
+    my $vtable  = [];
+    my $mmd     = [];
+    my $fh      = FileHandle->new( $file, O_RDONLY ) or die "Can't open $file for reading: $!\n";
     my $section = 'MAIN';
 
     my $default_attrs = {};
@@ -96,13 +94,13 @@ sub parse_vtable {
             m/^\s*
             ($type_re)\s+
             ($ident_re)\s*
-        \(($arglist_re)\)
-        (?:\s+(MMD_\w+))?\s*($attrs_re)$/x
+            \(($arglist_re)\)
+            (?:\s+(MMD_\w+))?\s*($attrs_re)$/x
             )
         {
             my $mmdop = defined $4 ? $4 : -1;
-
             my $entry = [ $1, $2, $3, $section, $mmdop, parse_attrs( $5, $default_attrs ) ];
+
             if (defined $4) {
                 push @{$mmd}, $entry;
             }
