@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2004-2007, The Perl Foundation.
-$Id: ascii.c 20252 2007-07-27 03:33:24Z petdance $
+$Id: ascii.c 21286 2007-09-14 18:49:07Z particle $
 
 =head1 NAME
 
@@ -243,7 +243,7 @@ upcase(PARROT_INTERP, NOTNULL(STRING *source_string))
         UINTVAL offset;
 
         for (offset = 0; offset < n; offset++) {
-            buffer[offset] = toupper((unsigned char)buffer[offset]);
+            buffer[offset] = (char)toupper(buffer[offset]);
         }
     }
 }
@@ -259,7 +259,7 @@ downcase(PARROT_INTERP, NOTNULL(STRING *source_string))
         UINTVAL offset;
 
         for (offset = 0; offset < n; offset++) {
-            buffer[offset] = tolower((unsigned char)buffer[offset]);
+            buffer[offset] = (char)tolower(buffer[offset]);
         }
     }
 }
@@ -274,9 +274,9 @@ titlecase(PARROT_INTERP, NOTNULL(STRING *source_string))
         char * const buffer = source_string->strstart;
         UINTVAL offset;
 
-        buffer[0] = toupper((unsigned char)buffer[0]);
+        buffer[0] = (char)toupper(buffer[0]);
         for (offset = 1; offset < n; offset++) {
-            buffer[offset] = tolower((unsigned char)buffer[offset]);
+            buffer[offset] = (char)tolower(buffer[offset]);
         }
     }
 }
@@ -288,7 +288,7 @@ upcase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
 
     if (source_string->strlen) {
         char * const buffer = source_string->strstart;
-        buffer[0] = toupper((unsigned char)buffer[0]);
+        buffer[0] = (char)toupper(buffer[0]);
     }
 }
 
@@ -299,7 +299,7 @@ downcase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
 
     if (source_string->strlen) {
         char * const buffer = source_string->strstart;
-        buffer[0] = tolower((unsigned char)buffer[0]);
+        buffer[0] = (char)tolower(buffer[0]);
     }
 }
 
@@ -310,7 +310,7 @@ titlecase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
 
     if (source_string->strlen) {
         char * const buffer = source_string->strstart;
-        buffer[0] = toupper((unsigned char)buffer[0]);
+        buffer[0] = (char)toupper(buffer[0]);
     }
 }
 
