@@ -1,7 +1,7 @@
 /* ascii.h
  *  Copyright (C) 2004-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: ascii.h 19914 2007-07-16 21:06:48Z petdance $
+ *     $Id: ascii.h 20003 2007-07-19 03:33:27Z petdance $
  *  Overview:
  *     This is the header for the ascii charset functions
  *  Data Structure and Algorithms:
@@ -53,18 +53,22 @@ INTVAL ascii_cs_rindex( PARROT_INTERP,
 
 PARROT_WARN_UNUSED_RESULT
 STRING * ascii_get_graphemes( PARROT_INTERP,
-    STRING *source_string,
+    NOTNULL(STRING *source_string),
     UINTVAL offset,
     UINTVAL count )
-        __attribute__nonnull__(1);
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 STRING * ascii_get_graphemes_inplace( PARROT_INTERP,
-    STRING *source_string,
+    NOTNULL(STRING *source_string),
     UINTVAL offset,
     UINTVAL count,
-    STRING *dest_string )
-        __attribute__nonnull__(1);
+    NOTNULL(STRING *dest_string) )
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(5);
 
 STRING * charset_cvt_ascii_to_binary( PARROT_INTERP,
     NOTNULL(STRING *src),
@@ -87,7 +91,8 @@ INTVAL mixed_cs_index( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
-CHARSET * Parrot_charset_ascii_init( PARROT_INTERP )
+PARROT_CANNOT_RETURN_NULL
+const CHARSET * Parrot_charset_ascii_init( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 /* HEADERIZER END: src/charset/ascii.c */
