@@ -1,7 +1,7 @@
 /* binary.h
- *  Copyright (C) 2004, The Perl Foundation.
+ *  Copyright (C) 2004-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: binary.h 19796 2007-07-11 22:09:12Z petdance $
+ *     $Id: binary.h 20003 2007-07-19 03:33:27Z petdance $
  *  Overview:
  *     This is the header for the binary charset functions
  *  Data Structure and Algorithms:
@@ -15,7 +15,8 @@
 
 /* HEADERIZER BEGIN: src/charset/binary.c */
 
-CHARSET * Parrot_charset_binary_init( PARROT_INTERP )
+PARROT_CANNOT_RETURN_NULL
+const CHARSET * Parrot_charset_binary_init( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 /* HEADERIZER END: src/charset/binary.c */
