@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2004-2007, The Perl Foundation.
-$Id: iso-8859-1.c 20342 2007-07-30 05:57:58Z petdance $
+$Id: iso-8859-1.c 21286 2007-09-14 18:49:07Z particle $
 
 =head1 NAME
 
@@ -227,8 +227,8 @@ upcase(PARROT_INTERP, NOTNULL(STRING *source_string))
         if (c >= 0xe0 && c != 0xf7)
             c &= ~0x20;
         else
-            c = toupper((unsigned char)c);
-        buffer[offset] = c;
+            c = toupper(c);
+        buffer[offset] = (unsigned char)c;
     }
 }
 
@@ -246,8 +246,8 @@ downcase(PARROT_INTERP, NOTNULL(STRING *source_string))
             if (c >= 0xc0 && c != 0xd7 && c <= 0xde)
                 c |= 0x20;
             else
-                c = tolower((unsigned char)c);
-            buffer[offset] = c;
+                c = tolower(c);
+            buffer[offset] = (unsigned char)c;
         }
     }
 }
@@ -268,16 +268,16 @@ titlecase(PARROT_INTERP, NOTNULL(STRING *source_string))
     if (c >= 0xe0 && c != 0xf7)
         c &= ~0x20;
     else
-        c = toupper((unsigned char)c);
-    buffer[0] = c;
+        c = toupper(c);
+    buffer[0] = (unsigned char)c;
 
     for (offset = 1; offset < source_string->strlen; offset++) {
         c = buffer[offset];
         if (c >= 0xc0 && c != 0xd7 && c <= 0xde)
             c |= 0x20;
         else
-            c = tolower((unsigned char)c);
-        buffer[offset] = c;
+            c = tolower(c);
+        buffer[offset] = (unsigned char)c;
     }
 }
 
@@ -294,8 +294,8 @@ upcase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
         if (c >= 0xe0 && c != 0xf7)
             c &= ~0x20;
         else
-            c = toupper((unsigned char)c);
-        buffer[0] = c;
+            c = toupper(c);
+        buffer[0] = (unsigned char)c;
     }
 }
 
@@ -312,9 +312,8 @@ downcase_first(PARROT_INTERP, NOTNULL(STRING *source_string))
         if (c >= 0xc0 && c != 0xd7 && c <= 0xde)
             c &= ~0x20;
         else
-            c = tolower((unsigned char)c);
-        buffer[0] = c;
-        buffer[0] = toupper((unsigned char)buffer[0]);
+            c = tolower(c);
+        buffer[0] = (unsigned char)c;
     }
 }
 
