@@ -1,7 +1,7 @@
 /* iso_8859_1.h
  *  Copyright (C) 2004-2007, The Perl Foundation.
  *  SVN Info
- *     $Id: iso-8859-1.h 19882 2007-07-14 22:33:30Z petdance $
+ *     $Id: iso-8859-1.h 20003 2007-07-19 03:33:27Z petdance $
  *  Overview:
  *     This is the header for the iso_8859-1 charset functions
  *  Data Structure and Algorithms:
@@ -22,7 +22,8 @@ STRING * charset_cvt_iso_8859_1_to_ascii( PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-CHARSET * Parrot_charset_iso_8859_1_init( PARROT_INTERP )
+PARROT_CANNOT_RETURN_NULL
+const CHARSET * Parrot_charset_iso_8859_1_init( PARROT_INTERP )
         __attribute__nonnull__(1);
 
 /* HEADERIZER END: src/charset/iso-8859-1.c */
