@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: debug.c 20488 2007-08-05 11:26:43Z paultcochrane $
+$Id: debug.c 20884 2007-08-28 14:25:19Z paultcochrane $
 
 =head1 NAME
 
@@ -925,7 +925,12 @@ PDB_set_break(PARROT_INTERP, NOTNULL(const char *command))
     /* Allocate the new break point */
     newbreak = mem_allocate_typed(PDB_breakpoint_t);
 
-    skip_command(command);
+    if (command) {
+        skip_command(command);
+    }
+    else {
+        real_exception(interp, NULL, 1, "NULL command passed to PDB_set_break");
+    }
     condition = NULL;
 
     /* if there is another argument to break, besides the line number,
