@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2006, The Perl Foundation.
-$Id: dynext.c 20709 2007-08-19 07:03:58Z chromatic $
+$Id: dynext.c 21355 2007-09-18 19:40:14Z chromatic $
 
 =head1 NAME
 
@@ -142,7 +142,7 @@ is_loaded(PARROT_INTERP, NOTNULL(STRING *path))
     PMC * const dyn_libs = VTABLE_get_pmc_keyed_int(interp, iglobals,
             IGLOBALS_DYN_LIBS);
     if (!VTABLE_exists_keyed_str(interp, dyn_libs, path))
-        return NULL;
+        return PMCNULL;
     return VTABLE_get_pmc_keyed_str(interp, dyn_libs, path);
 }
 
@@ -235,7 +235,7 @@ get_path(PARROT_INTERP, NOTNULL(STRING *lib), NOTNULL(void **handle),
      * [shouldn't this happen in Parrot_locate_runtime_file instead?]
      */
 #ifdef WIN32
-    if (memcmp(lib->strstart, "lib", 3) == 0) {
+    if (!STRING_IS_EMPTY(lib) && memcmp(lib->strstart, "lib", 3) == 0) {
         *handle = Parrot_dlopen((char*)lib->strstart + 3);
         if (*handle) {
             path = string_substr(interp, lib, 3, lib->strlen - 3, NULL, 0);
@@ -463,12 +463,13 @@ Parrot_load_lib(PARROT_INTERP, NULLOK(STRING *lib), SHIM(PMC *initializer))
     if (lib == NULL) {
         wo_ext   = string_from_literal(interp, "");
         lib_name = NULL;
+        ext      = NULL;
     }
     else {
         lib_name = parrot_split_path_ext(interp, lib, &wo_ext, &ext);
     }
     lib_pmc = is_loaded(interp, wo_ext);
-    if (lib_pmc) {
+    if (!PMC_IS_NULL(lib_pmc)) {
         /* UNLOCK() */
         return lib_pmc;
     }
