@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: exit.c 19881 2007-07-14 22:05:00Z petdance $
+$Id: exit.c 20539 2007-08-07 16:56:38Z particle $
 
 =head1 NAME
 
@@ -31,8 +31,8 @@ Register the specified function to be called on exit.
 */
 
 PARROT_API
-int
-Parrot_on_exit(PARROT_INTERP, NOTNULL(exit_handler_f function), void *arg)
+void
+Parrot_on_exit(PARROT_INTERP, NOTNULL(exit_handler_f function), NULLOK(void *arg))
 {
     /* XXX  we might want locking around the list access.   I'm sure this
      * will be the least of the threading issues. */
@@ -43,7 +43,6 @@ Parrot_on_exit(PARROT_INTERP, NOTNULL(exit_handler_f function), void *arg)
     new_node->arg = arg;
     new_node->next = interp->exit_handler_list;
     interp->exit_handler_list = new_node;
-    return 0;
 }
 
 /*
@@ -72,6 +71,10 @@ Parrot_exit(PARROT_INTERP, int status)
      *      (Parrot_really_destroy) has run
      */
     handler_node_t *node = interp->exit_handler_list;
+
+    Parrot_block_DOD(interp);
+    Parrot_block_GC(interp);
+
     while (node) {
         handler_node_t * const next = node->next;
 
@@ -79,7 +82,6 @@ Parrot_exit(PARROT_INTERP, int status)
         mem_sys_free(node);
         node = next;
     }
-
     exit(status);
 }
 
