@@ -1,10 +1,10 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: dod.c 20687 2007-08-18 20:16:33Z chromatic $
+$Id: dod.c 21356 2007-09-18 19:41:30Z chromatic $
 
 =head1 NAME
 
-src/dod.c - Dead object destruction of the various headers
+src/gc/dod.c - Dead object destruction of the various headers
 
 =head1 DESCRIPTION
 
@@ -18,6 +18,10 @@ There's also a verbose mode for garbage collection.
 
 =head1 FUNCTIONS
 
+=over 4
+
+=cut
+
 */
 
 #define DOD_C_SOURCE
@@ -65,7 +69,7 @@ int CONSERVATIVE_POINTER_CHASING = 0;
 
 /*
 
-FUNCDOC: mark_special
+=item C<mark_special>
 
 Mark a special PMC. If it has a C<PMC_EXT> structure, append or prepend
 the C<next_for_GC> pointer; otherwise, do the custom mark directly.
@@ -73,6 +77,8 @@ the C<next_for_GC> pointer; otherwise, do the custom mark directly.
 This should really be inline, so if inline isn't available, it would
 be better if it were a macro.
 
+=cut
+
 */
 
 static void
@@ -153,6 +159,16 @@ mark_special(PARROT_INTERP, NOTNULL(PMC *obj))
     }
 }
 
+/*
+
+=item C<pobject_lives>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
 PARROT_API
 void
 pobject_lives(PARROT_INTERP, NOTNULL(PObj *obj))
@@ -221,7 +237,7 @@ pobject_lives(PARROT_INTERP, NOTNULL(PObj *obj))
 
 /*
 
-FUNCDOC: Parrot_dod_trace_root
+=item C<Parrot_dod_trace_root>
 
 Trace the root set. Returns 0 if it's a lazy DOD run and all objects
 that need timely destruction were found.
@@ -232,9 +248,10 @@ C<trace_stack> can have these values:
  1 ... trace whole root set
  2 ... trace system areas only
 
+=cut
+
 */
 
-PARROT_API
 int
 Parrot_dod_trace_root(PARROT_INTERP, int trace_stack)
 {
@@ -278,18 +295,7 @@ Parrot_dod_trace_root(PARROT_INTERP, int trace_stack)
      * It seems that the Class PMC gets DODed - these should
      * get created as constant PMCs.
      */
-    for (i = 1; i < (unsigned int)interp->n_vtable_max; i++) {
-        const VTABLE * const vtable = interp->vtables[i];
-        /*
-         * XXX dynpmc groups have empty slots for abstract objects
-         */
-        if (vtable) {
-            if (vtable->mro)
-                pobject_lives(interp, (PObj *)vtable->mro);
-
-            pobject_lives(interp, (PObj *)vtable->_namespace);
-        }
-    }
+    mark_vtables(interp);
 
     /* mark exception list */
     for (i = 0; i <= E_LAST_PYTHON_E; ++i)
@@ -340,11 +346,13 @@ Parrot_dod_trace_root(PARROT_INTERP, int trace_stack)
 
 /*
 
-FUNCDOC: trace_active_PMCs
+=item C<trace_active_PMCs>
 
 Do a full trace run and mark all the PMCs as active if they are. Returns
 whether the run completed, that is, whether it's safe to proceed with GC.
 
+=cut
+
 */
 
 static int
@@ -360,13 +368,14 @@ trace_active_PMCs(PARROT_INTERP, int trace_stack)
 
 /*
 
-FUNCDOC: Parrot_dod_trace_children
+=item C<Parrot_dod_trace_children>
 
 Returns whether the tracing process completed.
 
+=cut
+
 */
 
-PARROT_API
 int
 Parrot_dod_trace_children(PARROT_INTERP, size_t how_many)
 {
@@ -416,18 +425,8 @@ Parrot_dod_trace_children(PARROT_INTERP, size_t how_many)
          * largest percentage of PMCs won't have anything in their data
          * pointer that we need to trace. */
         if (bits) {
-            if (bits == PObj_data_is_PMC_array_FLAG) {
-                /* malloced array of PMCs */
-                PMC ** const data = PMC_data_typed(current, PMC **);
-
-                if (data) {
-                    int i;
-                    for (i = 0; i < PMC_int_val(current); i++) {
-                        if (data[i])
-                            pobject_lives(interp, (PObj *)data[i]);
-                    }
-                }
-            }
+            if (bits == PObj_data_is_PMC_array_FLAG)
+                Parrot_dod_trace_pmc_data(interp, current);
             else {
                 /* All that's left is the custom */
                 PARROT_ASSERT(!PObj_on_free_list_TEST(current));
@@ -455,15 +454,42 @@ Parrot_dod_trace_children(PARROT_INTERP, size_t how_many)
     return 1;
 }
 
+/*
+
+=item C<Parrot_dod_trace_pmc_data>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
+void
+Parrot_dod_trace_pmc_data(PARROT_INTERP, NOTNULL(PMC * const p))
+{
+    /* malloced array of PMCs */
+    PMC ** const data = PMC_data_typed(p, PMC **);
+    INTVAL i;
+
+    if (!data)
+        return;
+
+    for (i = 0; i < PMC_int_val(p); i++) {
+        if (data[i])
+            pobject_lives(interp, (PObj *)data[i]);
+    }
+}
 
 #ifdef GC_IS_MALLOC
 
 /*
 
-FUNCDOC: clear_cow
+=item C<clear_cow>
 
 Clear the COW ref count.
 
+=cut
+
 */
 
 void
@@ -503,10 +529,12 @@ clear_cow(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int cleanup)
 
 /*
 
-FUNCDOC: used_cow
+=item C<used_cow>
 
 Find other users of COW's C<bufstart>.
 
+=cut
+
 */
 
 void
@@ -543,25 +571,25 @@ used_cow(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int cleanup)
 
 /*
 
-FUNCDOC: Parrot_dod_sweep
+=item C<Parrot_dod_sweep>
 
 Put any buffers/PMCs that are now unused onto the pool's free list. If
 C<GC_IS_MALLOC>, bufstart gets freed too, if possible. Avoid buffers that
 are immune from collection (i.e. constant).
 
+=cut
+
 */
 
-PARROT_API
 void
 Parrot_dod_sweep(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
 {
-    Arenas * const arena_base = interp->arena_base;
-
     UINTVAL i;
     UINTVAL total_used    = 0;
     UINTVAL object_size   = pool->object_size;
 
     Small_Object_Arena *cur_arena;
+    dod_object_fn_type dod_object = pool->dod_object;
 
 #if GC_VERBOSE
     if (Interp_trace_TEST(interp, 1)) {
@@ -617,104 +645,179 @@ Parrot_dod_sweep(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool))
                     }
                 }
 
-                /* if object is a PMC and needs destroying */
-                if (PObj_is_PMC_TEST(b)) {
-                    PMC * const p = (PMC *)b;
+                assert(dod_object);
+                dod_object(interp, pool, b);
 
-                    /* then destroy it here
-                     *
-                     * TODO collect objects with finalizers
-                    */
-                    if (PObj_needs_early_DOD_TEST(p))
-                        --arena_base->num_early_DOD_PMCs;
-
-                    if (PObj_active_destroy_TEST(p))
-                        VTABLE_destroy(interp, p);
-
-                    if (PObj_is_PMC_EXT_TEST(p) && p->pmc_ext != NULL) {
-                        /* if the PMC has a PMC_EXT structure,
-                         * return it to the pool/arena */
-                        Small_Object_Pool * const ext_pool
-                            = arena_base->pmc_ext_pool;
-
-                        if (PObj_is_PMC_shared_TEST(p) && PMC_sync(p)) {
-                            MUTEX_DESTROY(PMC_sync(p)->pmc_lock);
-                            mem_internal_free(PMC_sync(p));
-                            PMC_sync(p) = NULL;
-                        }
-
-                        ext_pool->add_free_object(interp, ext_pool, (PObj *)p->pmc_ext);
-                    }
-#ifndef NDEBUG
-                    /*
-                     * invalidate the PMC
-                     */
-                    p->pmc_ext     = (PMC_EXT *)0xdeadbeef;
-                    p->vtable      = (VTABLE  *)0xdeadbeef;
-                    PMC_pmc_val(p) = (PMC     *)0xdeadbeef;
-#endif
-                }
-                /* else object is a buffer(like) */
-                else if (PObj_sysmem_TEST(b) && PObj_bufstart(b)) {
-                    /* has sysmem allocated, e.g. string_pin */
-                    mem_sys_free(PObj_bufstart(b));
-                    PObj_bufstart(b) = NULL;
-                    PObj_buflen(b)   = 0;
-                }
-                else {
-#ifdef GC_IS_MALLOC
-                    /* free allocated space at (int*)bufstart - 1,
-                     * but not if it used COW or is external
-                     */
-                    if (PObj_bufstart(b) &&
-                            !PObj_is_external_or_free_TESTALL(b)) {
-                        if (PObj_COW_TEST(b)) {
-                            INTVAL *refcount = ((INTVAL *)PObj_bufstart(b) - 1);
-
-                            if (!--(*refcount)) {
-                                free(refcount); /* the actual bufstart */
-                                refcount = NULL;
-                            }
-                        }
-                        else
-                            free((INTVAL*)PObj_bufstart(b) - 1);
-                    }
-#else
-                    /*
-                     * XXX Jarkko did report that on irix pool->mem_pool
-                     *     was NULL, which really shouldn't happen
-                     */
-                    if (pool->mem_pool) {
-                        if (!PObj_COW_TEST(b)) {
-                            ((Memory_Pool *)
-                             pool->mem_pool)->guaranteed_reclaimable +=
-                                PObj_buflen(b);
-                        }
-                        ((Memory_Pool *)
-                         pool->mem_pool)->possibly_reclaimable +=
-                            PObj_buflen(b);
-                    }
-#endif
-                    PObj_buflen(b) = 0;
-                }
-                PObj_flags_SETTO((PObj *)b, PObj_on_free_list_FLAG);
                 pool->add_free_object(interp, pool, b);
             }
 next:
             b = (Buffer *)((char *)b + object_size);
         }
     }
+
     pool->num_free_objects = pool->total_objects - total_used;
 }
 
+/*
+
+=item C<Parrot_dod_free_pmc>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
+void
+Parrot_dod_free_pmc(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool),
+    NOTNULL(PObj *p))
+{
+    PMC           *pmc        = (PMC *)p;
+    Arenas * const arena_base = interp->arena_base;
+
+    /* TODO collect objects with finalizers */
+    if (PObj_needs_early_DOD_TEST(p))
+        --arena_base->num_early_DOD_PMCs;
+
+    if (PObj_active_destroy_TEST(p))
+        VTABLE_destroy(interp, pmc);
+
+    if (PObj_is_PMC_EXT_TEST(p))
+         Parrot_free_pmc_ext(interp, pmc);
+
+#ifndef NDEBUG
+
+    pmc->pmc_ext     = (PMC_EXT *)0xdeadbeef;
+    pmc->vtable      = (VTABLE  *)0xdeadbeef;
+    PMC_pmc_val(pmc) = (PMC     *)0xdeadbeef;
+
+#endif
+
+}
+
+/*
+
+=item C<Parrot_free_pmc_ext>
+
+Frees the PMC_EXT structure attached to a PMC, if it exists.
+
+=cut
+
+*/
+
+void
+Parrot_free_pmc_ext(PARROT_INTERP, NOTNULL(PMC *p))
+{
+    /* if the PMC has a PMC_EXT structure, return it to the pool/arena */
+    Arenas            * const arena_base = interp->arena_base;
+    Small_Object_Pool * const ext_pool   = arena_base->pmc_ext_pool;
+
+    if (PObj_is_PMC_shared_TEST(p) && PMC_sync(p)) {
+        MUTEX_DESTROY(PMC_sync(p)->pmc_lock);
+        mem_internal_free(PMC_sync(p));
+        PMC_sync(p) = NULL;
+    }
+
+    if (p->pmc_ext)
+        ext_pool->add_free_object(interp, ext_pool, (PObj *)p->pmc_ext);
+
+    p->pmc_ext = NULL;
+}
+
+/*
+
+=item C<parrot_dod_free_sysmem>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
+void
+Parrot_dod_free_sysmem(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool),
+    NOTNULL(PObj *b))
+{
+    /* has sysmem allocated, e.g. string_pin */
+    if (PObj_sysmem_TEST(b) && PObj_bufstart(b))
+        mem_sys_free(PObj_bufstart(b));
+
+    PObj_bufstart(b) = NULL;
+    PObj_buflen(b)   = 0;
+}
+
+/*
+
+=item C<Parrot_dod_free_buffer_malloc>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
+void
+Parrot_dod_free_buffer_malloc(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool),
+    NOTNULL(PObj *b))
+{
+
+    /* free allocated space at (int *)bufstart - 1, but not if it used COW or is
+     * external */
+    PObj_buflen(b) = 0;
+
+    if (!PObj_bufstart(b) || PObj_is_external_or_free_TESTALL(b))
+        return;
+
+    if (PObj_COW_TEST(b)) {
+        INTVAL *refcount = PObj_bufrefcountptr(b);
+
+        if (!--(*refcount)) {
+            free(refcount); /* the actual bufstart */
+            refcount = NULL;
+        }
+    }
+    else
+        free(PObj_bufrefcountptr(b));
+}
+
+/*
+
+=item C<Parrot_dod_free_buffer>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
+void
+Parrot_dod_free_buffer( PARROT_INTERP, NOTNULL(Small_Object_Pool *pool),
+    NOTNULL(PObj *b))
+{
+    Memory_Pool *mem_pool = (Memory_Pool *)pool->mem_pool;
+
+    /* XXX Jarkko reported that on irix pool->mem_pool was NULL, which really
+     * shouldn't happen */
+    if (mem_pool) {
+        if (!PObj_COW_TEST(b))
+            mem_pool->guaranteed_reclaimable += PObj_buflen(b);
+
+         mem_pool->possibly_reclaimable += PObj_buflen(b);
+    }
+
+    PObj_buflen(b)        = 0;
+}
+
 #ifndef PLATFORM_STACK_WALK
 
 /*
 
-FUNCDOC: find_common_mask
+=item C<find_common_mask>
 
 Find a mask covering the longest common bit-prefix of C<val1> and C<val2>.
 
+=cut
+
 */
 
 PARROT_CONST_FUNCTION
@@ -746,10 +849,12 @@ find_common_mask(PARROT_INTERP, size_t val1, size_t val2)
 
 /*
 
-FUNCDOC: trace_mem_block
+=item C<trace_mem_block>
 
 Traces the memory block between C<lo_var_ptr> and C<hi_var_ptr>.
 
+=cut
+
 */
 
 void
@@ -813,9 +918,12 @@ trace_mem_block(PARROT_INTERP, size_t lo_var_ptr, size_t hi_var_ptr)
 
 /*
 
-FUNCDOC:
+=item C<clear_live_bits>
+
 Run through all PMC arenas and clear live bits.
 
+=cut
+
 */
 
 static void
@@ -836,7 +944,16 @@ clear_live_bits(NOTNULL(Small_Object_Pool *pool))
 
 }
 
-PARROT_API
+/*
+
+=item C<Parrot_dod_clear_live_bits>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
 void
 Parrot_dod_clear_live_bits(PARROT_INTERP)
 {
@@ -846,13 +963,14 @@ Parrot_dod_clear_live_bits(PARROT_INTERP)
 
 /*
 
-FUNCDOC: Parrot_dod_profile_start
+=item C<Parrot_dod_profile_start>
 
 Records the start time of a DOD run when profiling is enabled.
 
+=cut
+
 */
 
-PARROT_API
 void
 Parrot_dod_profile_start(PARROT_INTERP)
 {
@@ -862,14 +980,15 @@ Parrot_dod_profile_start(PARROT_INTERP)
 
 /*
 
-FUNCDOC: Parrot_dod_profile_end
+=item C<Parrot_dod_profile_end>
 
 Records the end time of the DOD part C<what> run when profiling is enabled.
 Also record start time of next part.
 
+=cut
+
 */
 
-PARROT_API
 void
 Parrot_dod_profile_end(PARROT_INTERP, int what)
 {
@@ -895,13 +1014,14 @@ Parrot_dod_profile_end(PARROT_INTERP, int what)
 
 /*
 
-FUNCDOC: Parrot_dod_ms_run_init
+=item C<Parrot_dod_ms_run_init>
 
 Prepare for a mark & sweep DOD run.
 
+=cut
+
 */
 
-PARROT_API
 void
 Parrot_dod_ms_run_init(PARROT_INTERP)
 {
@@ -913,6 +1033,16 @@ Parrot_dod_ms_run_init(PARROT_INTERP)
     arena_base->num_extended_PMCs   = 0;
 }
 
+/*
+
+=item C<sweep_cb>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
 static int
 sweep_cb(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag,
     NOTNULL(void *arg))
@@ -941,13 +1071,14 @@ sweep_cb(PARROT_INTERP, NOTNULL(Small_Object_Pool *pool), int flag,
 
 /*
 
-FUNCDOC: Parrot_dod_ms_run
+=item C<Parrot_dod_ms_run>
 
 Run the stop-the-world mark & sweep collector.
 
+=cut
+
 */
 
-PARROT_API
 void
 Parrot_dod_ms_run(PARROT_INTERP, int flags)
 {
@@ -1030,13 +1161,14 @@ Parrot_dod_ms_run(PARROT_INTERP, int flags)
 
 /*
 
-FUNCDOC: Parrot_do_dod_run
+=item C<Parrot_do_dod_run>
 
 Call the configured garbage collector to reclaim unused headers.
 
+=cut
+
 */
 
-PARROT_API
 void
 Parrot_do_dod_run(PARROT_INTERP, UINTVAL flags)
 {
@@ -1046,6 +1178,8 @@ Parrot_do_dod_run(PARROT_INTERP, UINTVAL flags)
 
 /*
 
+=back
+
 =head1 SEE ALSO
 
 F<include/parrot/dod.h>, F<src/cpu_dep.c>, F<docs/dev/dod.dev> and
@@ -1055,6 +1189,8 @@ F<docs/pdds/pdd09_gc.pod>.
 
 Initial version by Mike Lambert on 2002.05.27.
 
+=cut
+
 */
 
 /*
