@@ -1,10 +1,10 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: memory.c 20015 2007-07-19 16:11:36Z petdance $
+$Id: memory.c 21346 2007-09-18 14:44:10Z paultcochrane $
 
 =head1 NAME
 
-src/memory.c - Memory allocation
+src/gc/memory.c - Memory allocation
 
 =head1 DESCRIPTION
 
@@ -15,6 +15,10 @@ setup function to initialize the memory pools.
 
 =head2 Functions
 
+=over 4
+
+=cut
+
 */
 
 #include "parrot/parrot.h"
@@ -24,10 +28,12 @@ setup function to initialize the memory pools.
 
 /*
 
-FUNCDOC: mem_sys_allocate
+=item C<mem_sys_allocate>
 
 Uses C<malloc> to allocate system memory.
 
+=cut
+
 */
 
 PARROT_API
@@ -45,6 +51,16 @@ mem_sys_allocate(size_t size)
     return ptr;
 }
 
+/*
+
+=item C<mem__internal_allocate>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
 PARROT_MALLOC
 PARROT_CANNOT_RETURN_NULL
 void *
@@ -65,10 +81,12 @@ mem__internal_allocate(size_t size, NOTNULL(const char *file), int line)
 
 /*
 
-FUNCDOC: mem_sys_allocate_zeroed
+=item C<mem_sys_allocate_zeroed>
 
 Uses C<calloc> to allocate system memory.
 
+=cut
+
 */
 
 PARROT_API
@@ -86,6 +104,16 @@ mem_sys_allocate_zeroed(size_t size)
     return ptr;
 }
 
+/*
+
+=item C<mem__internal_allocate_zeroed>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
 PARROT_MALLOC
 PARROT_CANNOT_RETURN_NULL
 void *
@@ -106,10 +134,12 @@ mem__internal_allocate_zeroed(size_t size, NOTNULL(const char *file), int line)
 
 /*
 
-FUNCDOC: mem_sys_realloc
+=item C<mem_sys_realloc>
 
 Resize a chunk of system memory.
 
+=cut
+
 */
 
 PARROT_API
@@ -134,10 +164,12 @@ mem__sys_realloc(NULLOK(void *from), size_t size)
 
 /*
 
-FUNCDOC: mem_sys_realloc_zeroed
+=item C<mem_sys_realloc_zeroed>
 
 Resize a chunk of system memory. Fill the newly allocated space with zeroes.
 
+=cut
+
 */
 
 PARROT_API
@@ -163,6 +195,16 @@ mem__sys_realloc_zeroed(NULLOK(void *from), size_t size, size_t old_size)
     return ptr;
 }
 
+/*
+
+=item C<mem__internal_realloc>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
 PARROT_MALLOC
 PARROT_CANNOT_RETURN_NULL
 void *
@@ -186,10 +228,12 @@ mem__internal_realloc(NOTNULL(void *from), size_t size,
 
 /*
 
-FUNCDOC: mem_sys_free
+=item C<mem_sys_free>
 
 Free a chunk of memory back to the system.
 
+=cut
+
 */
 
 PARROT_API
@@ -203,6 +247,16 @@ mem_sys_free(NULLOK(void *from))
         free(from);
 }
 
+/*
+
+=item C<mem__internal_free>
+
+TODO: Not yet documented!!!
+
+=cut
+
+*/
+
 void
 mem__internal_free(NULLOK(void *from), NOTNULL(const char *file), int line)
 {
@@ -217,10 +271,12 @@ mem__internal_free(NULLOK(void *from), NOTNULL(const char *file), int line)
 
 /*
 
-FUNCDOC: mem_setup_allocator
+=item C<mem_setup_allocator>
 
 Initializes the allocator.
 
+=cut
+
 */
 
 void
@@ -243,6 +299,14 @@ mem_setup_allocator(PARROT_INTERP)
     Parrot_initialize_header_pools(interp);
 }
 
+/*
+
+=back
+
+=cut
+
+*/
+
 /*
  * Local variables:
  *   c-file-style: "parrot"
