@@ -1,10 +1,10 @@
 /*
 Copyright (C) 2001-2006, The Perl Foundation.
-$Id: res_lea.c 20258 2007-07-27 08:00:07Z petdance $
+$Id: res_lea.c 21346 2007-09-18 14:44:10Z paultcochrane $
 
 =head1 NAME
 
-src/res_lea.c - Resource allocation using malloc
+src/gc/res_lea.c - Resource allocation using malloc
 
 =head1 DESCRIPTION
 
