@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: headers.c 20709 2007-08-19 07:03:58Z chromatic $
+$Id: headers.c 21098 2007-09-06 00:47:41Z chromatic $
 
 =head1 NAME
 
@@ -118,7 +118,9 @@ new_pmc_pool(PARROT_INTERP)
     Small_Object_Pool * const pmc_pool =
         new_small_object_pool(sizeof (PMC), num_headers);
 
-    pmc_pool->mem_pool = NULL;
+    pmc_pool->mem_pool   = NULL;
+    pmc_pool->dod_object = Parrot_dod_free_pmc;
+
     (interp->arena_base->init_pool)(interp, pmc_pool);
     return pmc_pool;
 }
@@ -143,7 +145,8 @@ new_bufferlike_pool(PARROT_INTERP, size_t actual_buffer_size)
     Small_Object_Pool * const pool =
             new_small_object_pool(buffer_size, num_headers);
 
-    pool->mem_pool = interp->arena_base->memory_pool;
+    pool->dod_object = Parrot_dod_free_sysmem;
+    pool->mem_pool   = interp->arena_base->memory_pool;
     (interp->arena_base->init_pool)(interp, pool);
     return pool;
 }
@@ -161,7 +164,15 @@ PARROT_CANNOT_RETURN_NULL
 Small_Object_Pool *
 new_buffer_pool(PARROT_INTERP)
 {
-    return make_bufferlike_pool(interp, sizeof (Buffer));
+    Small_Object_Pool *pool = make_bufferlike_pool(interp, sizeof (Buffer));
+
+#ifdef GC_IS_MALLOC
+    pool->dod_object        = Parrot_dod_free_buffer_malloc;
+#else
+    pool->dod_object        = Parrot_dod_free_buffer;
+#endif
+
+    return pool;
 }
 
 /*
@@ -731,11 +742,11 @@ Parrot_destroy_header_pools(PARROT_INTERP)
 #endif
     ignored = Parrot_forall_header_pools(interp, POOL_PMC | POOL_CONST, NULL,
             sweep_cb_pmc);
+    UNUSED(ignored);
 
     for (pass = start; pass <= 2; pass++) {
         ignored = Parrot_forall_header_pools(interp, POOL_BUFFER | POOL_CONST,
                 (void *)pass, sweep_cb_buf);
-
     }
 
     free_pool(interp->arena_base->pmc_ext_pool);
@@ -816,7 +827,8 @@ Parrot_merge_header_pools(NOTNULL(Interp *dest_interp), NOTNULL(Interp *source_i
         if (i >= dest_arena->num_sized ||
             !dest_arena->sized_header_pools[i]) {
             Small_Object_Pool *ignored = make_bufferlike_pool(dest_interp,
-                i * sizeof (void *) + sizeof (Buffer));
+                    i * sizeof (void *) + sizeof (Buffer));
+            UNUSED(ignored);
             PARROT_ASSERT(dest_arena->sized_header_pools[i]);
         }
 
