@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2005-2007, The Perl Foundation.
-$Id: hll.c 20751 2007-08-21 03:56:22Z chromatic $
+$Id: hll.c 21270 2007-09-14 01:15:24Z chromatic $
 
 =head1 NAME
 
@@ -17,7 +17,7 @@ feature.
    interp->HLL_info
 
    @HLL_info = [
-     [ hll_name, hll_lib, { core_type => HLL_type, ... }, namespace ],
+     [ hll_name, hll_lib, { core_type => HLL_type, ... }, namespace, hll_id ],
      ...
      ]
 
@@ -63,7 +63,7 @@ the context.  If no type is registered, returns C<core_type>.
 
 PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
-static PMC* new_hll_entry( PARROT_INTERP )
+static PMC* new_hll_entry( PARROT_INTERP, NULLOK(STRING *hll_name) )
         __attribute__nonnull__(1);
 
 /* HEADERIZER END: static */
@@ -88,9 +88,12 @@ static PMC* new_hll_entry( PARROT_INTERP )
 PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
 static PMC*
-new_hll_entry(PARROT_INTERP)
+new_hll_entry(PARROT_INTERP, NULLOK(STRING *entry_name))
 {
     PMC * const hll_info = interp->HLL_info;
+    INTVAL      id       = VTABLE_elements(interp, hll_info);
+    PMC        *entry_id;
+
     /*
      * ATT: all items that are owned by the HLL_info structure
      *      have to be created as constant objects, because
@@ -98,65 +101,40 @@ new_hll_entry(PARROT_INTERP)
      */
     PMC * const entry = constant_pmc_new(interp, enum_class_FixedPMCArray);
 
-    VTABLE_push_pmc(interp, hll_info, entry);
+    if (entry_name && !STRING_IS_EMPTY(entry_name))
+        VTABLE_set_pmc_keyed_str(interp, hll_info, entry_name, entry);
+    else
+        VTABLE_push_pmc(interp, hll_info, entry);
+
     VTABLE_set_integer_native(interp, entry, e_HLL_MAX);
 
+    entry_id = constant_pmc_new(interp, enum_class_Integer);
+    VTABLE_set_integer_native(interp, entry_id, id);
+    VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_id, entry_id);
+
     return entry;
 }
 
+void
+Parrot_init_HLL(PARROT_INTERP)
+{
+    interp->HLL_info      =
+        constant_pmc_new(interp, enum_class_OrderedHash);
+    interp->HLL_namespace =
+        constant_pmc_new(interp, enum_class_ResizablePMCArray);
+
+    Parrot_register_HLL(interp, const_string(interp, "parrot"));
+}
+
 PARROT_API
 INTVAL
-Parrot_register_HLL(PARROT_INTERP, NULLOK(STRING *hll_name), NULLOK(STRING *hll_lib))
+Parrot_register_HLL(PARROT_INTERP, NOTNULL(STRING *hll_name))
 {
     PMC *entry, *name, *type_hash, *ns_hash, *hll_info;
     INTVAL idx;
 
     /* TODO LOCK or disallow in threads */
 
-    if (!hll_name) {
-        INTVAL nelements, i;
-
-        /* .loadlib pragma */
-        hll_info = interp->HLL_info;
-
-        START_WRITE_HLL_INFO(interp, hll_info);
-
-        nelements = VTABLE_elements(interp, hll_info);
-
-        for (i = 0; i < nelements; ++i) {
-            STRING      *name;
-            PMC * const  entry    = VTABLE_get_pmc_keyed_int(interp, hll_info, i);
-            PMC * const  lib_name = VTABLE_get_pmc_keyed_int(interp, entry,
-                e_HLL_lib);
-
-            if (PMC_IS_NULL(lib_name))
-                continue;
-
-            name = VTABLE_get_string(interp, lib_name);
-
-            if (!string_equal(interp, name, hll_lib))
-                break;
-        }
-
-        if (i < nelements)
-            return i;
-
-        entry    = new_hll_entry(interp);
-
-        VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_name, PMCNULL);
-
-        /* register dynlib */
-        name    = constant_pmc_new_noinit(interp, enum_class_String);
-        ASSERT_CONST_STRING(hll_lib);
-
-        VTABLE_set_string_native(interp, name, hll_lib);
-        VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_lib, name);
-
-        END_WRITE_HLL_INFO(interp, hll_info);
-
-        return 0;
-    }
-
     idx = Parrot_get_HLL_id(interp, hll_name);
 
     if (idx >= 0)
@@ -167,7 +145,7 @@ Parrot_register_HLL(PARROT_INTERP, NULLOK(STRING *hll_name), NULLOK(STRING *hll_
     START_WRITE_HLL_INFO(interp, hll_info);
 
     idx      = VTABLE_elements(interp, hll_info);
-    entry    = new_hll_entry(interp);
+    entry    = new_hll_entry(interp, hll_name);
 
     /* register HLL name */
     name     = constant_pmc_new_noinit(interp, enum_class_String);
@@ -194,23 +172,10 @@ Parrot_register_HLL(PARROT_INTERP, NULLOK(STRING *hll_name), NULLOK(STRING *hll_
     /* register HLL lib */
     name = constant_pmc_new_noinit(interp, enum_class_String);
 
-    if (!hll_lib)
-        hll_lib = const_string(interp, "");
-
-    ASSERT_CONST_STRING(hll_lib);
-
-    VTABLE_set_string_native(interp, name, hll_lib);
-    VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_lib, name);
-
     /* create HLL typemap hash */
     type_hash = Parrot_new_INTVAL_hash(interp, PObj_constant_FLAG);
     VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_typemap, type_hash);
 
-    /* load lib */
-    if (string_length(interp, hll_lib)) {
-        PMC *ignored = Parrot_load_lib(interp, hll_lib, NULL);
-    }
-
     /* UNLOCK */
     END_WRITE_HLL_INFO(interp, hll_info);
 
@@ -218,35 +183,72 @@ Parrot_register_HLL(PARROT_INTERP, NULLOK(STRING *hll_name), NULLOK(STRING *hll_
 }
 
 PARROT_API
-PARROT_WARN_UNUSED_RESULT
 INTVAL
-Parrot_get_HLL_id(PARROT_INTERP, NULLOK(STRING *hll_name))
+Parrot_register_HLL_lib(PARROT_INTERP, NOTNULL(STRING *hll_lib))
 {
-    INTVAL i, nelements;
+    PMC   *hll_info = interp->HLL_info;
+    PMC   *entry, *name;
+    INTVAL nelements, i;
 
-    PMC * const hll_info = interp->HLL_info;
-
-    START_READ_HLL_INFO(interp, hll_info);
+    START_WRITE_HLL_INFO(interp, hll_info);
 
     nelements = VTABLE_elements(interp, hll_info);
 
     for (i = 0; i < nelements; ++i) {
         STRING      *name;
         PMC * const  entry    = VTABLE_get_pmc_keyed_int(interp, hll_info, i);
-        PMC * const  name_pmc = VTABLE_get_pmc_keyed_int(interp, entry,
-                e_HLL_name);
+        PMC * const  lib_name = VTABLE_get_pmc_keyed_int(interp, entry,
+            e_HLL_lib);
 
-        if (PMC_IS_NULL(name_pmc))
+        if (PMC_IS_NULL(lib_name))
             continue;
 
-        name = VTABLE_get_string(interp, name_pmc);
-        if (!string_equal(interp, name, hll_name))
+        name = VTABLE_get_string(interp, lib_name);
+
+        if (!string_equal(interp, name, hll_lib))
             break;
     }
 
+    if (i < nelements)
+        return i;
+
+    entry    = new_hll_entry(interp, NULL);
+
+    VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_name, PMCNULL);
+
+    /* register dynlib */
+    name    = constant_pmc_new_noinit(interp, enum_class_String);
+    ASSERT_CONST_STRING(hll_lib);
+
+    VTABLE_set_string_native(interp, name, hll_lib);
+    VTABLE_set_pmc_keyed_int(interp, entry, e_HLL_lib, name);
+
+    END_WRITE_HLL_INFO(interp, hll_info);
+
+    return 0;
+}
+
+PARROT_API
+PARROT_WARN_UNUSED_RESULT
+INTVAL
+Parrot_get_HLL_id(PARROT_INTERP, NULLOK(STRING *hll_name))
+{
+    PMC * const hll_info = interp->HLL_info;
+    INTVAL      i;
+
+    START_READ_HLL_INFO(interp, hll_info);
+
+    if (!hll_name || !VTABLE_exists_keyed_str(interp, hll_info, hll_name))
+        i = -1;
+    else {
+        PMC *entry    = VTABLE_get_pmc_keyed_str(interp, hll_info, hll_name);
+        PMC *entry_id = VTABLE_get_pmc_keyed_int(interp, entry, e_HLL_id);
+        i = VTABLE_get_integer(interp, entry_id);
+    }
+
     END_READ_HLL_INFO(interp, hll_info);
 
-    return i < nelements ? i : -1;
+    return i;
 }
 
 PARROT_WARN_UNUSED_RESULT
