@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: inter_call.c 20363 2007-07-31 00:37:25Z rgrjr $
+$Id: inter_call.c 20885 2007-08-28 14:40:52Z paultcochrane $
 
 =head1 NAME
 
@@ -496,7 +496,6 @@ Parrot_fetch_arg(PARROT_INTERP, NOTNULL(call_state *st))
     }
 
     real_exception(interp, NULL, 1, "invalid call state mode");
-    return 0;
 }
 
 
@@ -1464,6 +1463,10 @@ Parrot_PCCINVOKE(PARROT_INTERP, NULLOK(PMC* pmc), NOTNULL(STRING *method_name),
         arg_ret_cnt[seen_arrow]++;
         max_regs[REGNO_PMC]++;
     }
+    else {
+        real_exception(interp, NULL, 1,
+                "NULL PMC passed into Parrot_PCCINVOKE");
+    }
 
     /* first loop through signature to get sizing info */
     for (x = signature; *x != '\0'; x++) {
