@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2003, The Perl Foundation.
-$Id: inter_create.c 20709 2007-08-19 07:03:58Z chromatic $
+$Id: inter_create.c 21144 2007-09-08 19:31:26Z bernhard $
 
 =head1 NAME
 
@@ -169,7 +169,7 @@ make_interpreter(NULLOK(Interp *parent), INTVAL flags)
 
     /* initialize classes - this needs mmd func table */
     interp->HLL_info = NULL;
-    Parrot_init(interp);
+    init_world_once(interp);
 
     /* context data */
     /* Initialize interpreter's flags */
