@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: interpreter.c 20709 2007-08-19 07:03:58Z chromatic $
+$Id: interpreter.c 20825 2007-08-24 15:26:09Z particle $
 
 =head1 NAME
 
@@ -495,9 +495,7 @@ exec_init_prederef(PARROT_INTERP, void *prederef_arena)
     load_prederef(interp, PARROT_CGP_CORE);
 
     if (!interp->code->prederef.code) {
-        size_t N = interp->code->base.size; /* XXX Unused */
         void **temp = (void **)prederef_arena;
-        opcode_t *pc = interp->code->base.data; /* XXX unused */
 
         interp->code->prederef.code = temp;
         /* TODO */
