@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: io_buf.c 19914 2007-07-16 21:06:48Z petdance $
+$Id: io_buf.c 20694 2007-08-19 01:37:57Z chromatic $
 
 =head1 NAME
 
@@ -16,7 +16,6 @@ The "buf" layer of Parrot IO. Buffering and all the fun stuff.
 
 #include "parrot/parrot.h"
 #include "io_private.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: none */
 /* HEADERIZER BEGIN: static */
@@ -28,6 +27,8 @@ static INTVAL PIO_buf_close( PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static ParrotIO * PIO_buf_fdopen( PARROT_INTERP,
     NOTNULL(ParrotIOLayer *layer),
     PIOHANDLE fd,
@@ -116,7 +117,7 @@ static INTVAL PIO_buf_setlinebuf( PARROT_INTERP,
         __attribute__nonnull__(3);
 
 static PIOOFF_T PIO_buf_tell( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io) )
         __attribute__nonnull__(3);
 
@@ -301,22 +302,21 @@ The buffer layer's C<FDOpen> function.
 
 */
 
+PARROT_CAN_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static ParrotIO *
 PIO_buf_fdopen(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), PIOHANDLE fd, INTVAL flags)
 {
     ParrotIOLayer * const l = PIO_DOWNLAYER(layer);
     ParrotIO * const io = PIO_fdopen_down(interp, l, fd, flags);
 
-    if (!io) {
-        /* error creating IO stream */
-        return NULL;
+    if (io) {
+        if (io->flags & PIO_F_CONSOLE)
+            PIO_buf_setlinebuf(interp, l, io);
+        else
+            PIO_buf_setbuf(interp, l, io, PIO_UNBOUND);
     }
 
-    if (io->flags & PIO_F_CONSOLE)
-        PIO_buf_setlinebuf(interp, l, io);
-    else
-        PIO_buf_setbuf(interp, l, io, PIO_UNBOUND);
-
     return io;
 }
 
@@ -786,7 +786,7 @@ PIO_buf_seek(PARROT_INTERP, NOTNULL(ParrotIOLayer *l), NOTNULL(ParrotIO *io),
     if ((newpos < io->fpos - (io->b.next - io->b.startb))
         || (newpos >= io->fpos + (io->b.endb - io->b.next))) {
         PIO_buf_flush(interp, l, io);
-        PIO_seek_down(interp, PIO_DOWNLAYER(l), io, newpos, SEEK_SET);
+        newpos = PIO_seek_down(interp, PIO_DOWNLAYER(l), io, newpos, SEEK_SET);
     }
     else {
         io->b.next += newpos - io->fpos;
