@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: io_passdown.c 19881 2007-07-14 22:05:00Z petdance $
+$Id: io_passdown.c 20052 2007-07-21 04:41:00Z petdance $
 
 =head1 NAME
 
@@ -33,17 +33,16 @@ Returns C<NULL> if no implementation is found.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO *
-PIO_open_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(const char *name),
-              INTVAL flags)
+PIO_open_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer),
+        NOTNULL(const char *name), INTVAL flags)
 {
     while (layer) {
-        if (layer->api->Open) {
+        if (layer->api->Open)
             return layer->api->Open(interp, layer, name, flags);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return NULL;
 }
 
@@ -59,18 +58,16 @@ Returns C<NULL> if no implementation is found.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO *
 PIO_open_async_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(const char *name),
-                    NOTNULL(const char *mode), DummyCodeRef * dummy)
+        NOTNULL(const char *mode), NOTNULL(DummyCodeRef *dummy))
 {
     while (layer) {
-        if (layer->api->Open_ASync) {
-            return layer->api->Open_ASync(interp, layer, name, mode,
-                                          dummy);
-        }
+        if (layer->api->Open_ASync)
+            return layer->api->Open_ASync(interp, layer, name, mode, dummy);
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return NULL;
 }
 
@@ -86,31 +83,27 @@ Returns C<NULL> if no implementation is found.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO *
 PIO_fdopen_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), PIOHANDLE fd, INTVAL flags)
 {
     while (layer) {
-        if (layer->api->FDOpen) {
+        if (layer->api->FDOpen)
             return layer->api->FDOpen(interp, layer, fd, flags);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return NULL;
 }
 
 PARROT_WARN_UNUSED_RESULT
 size_t
-PIO_peek_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io, STRING ** buf)
+PIO_peek_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io), NOTNULL(STRING **buf))
 {
     while (layer) {
-        if (layer->api->Peek) {
+        if (layer->api->Peek)
             return layer->api->Peek(interp, layer, io, buf);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-
-    /* No layer found */
     return 0;
 }
 
@@ -128,15 +121,13 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_close_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io)
+PIO_close_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io))
 {
     while (layer) {
-        if (layer->api->Close) {
+        if (layer->api->Close)
             return layer->api->Close(interp, layer, io);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return -1;
 }
 
@@ -153,15 +144,14 @@ Returns C<0> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 size_t
-PIO_write_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io, STRING *s)
+PIO_write_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+        NOTNULL(STRING *s))
 {
     while (layer) {
-        if (layer->api->Write) {
+        if (layer->api->Write)
             return layer->api->Write(interp, layer, io, s);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return 0;
 }
 
@@ -178,16 +168,14 @@ Returns C<0> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 size_t
-PIO_write_async_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io,
-                     STRING *s, DummyCodeRef *dummy)
+PIO_write_async_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+        NOTNULL(STRING *s), NOTNULL(DummyCodeRef *dummy))
 {
     while (layer) {
-        if (layer->api->Write_ASync) {
+        if (layer->api->Write_ASync)
             return layer->api->Write_ASync(interp, layer, io, s, dummy);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return 0;
 }
 
@@ -204,15 +192,14 @@ Returns C<0> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 size_t
-PIO_read_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io, STRING ** buf)
+PIO_read_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+        NOTNULL(STRING **buf))
 {
     while (layer) {
-        if (layer->api->Read) {
+        if (layer->api->Read)
             return layer->api->Read(interp, layer, io, buf);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return 0;
 }
 
@@ -229,16 +216,14 @@ Returns C<0> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 size_t
-PIO_read_async_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io,
-                    STRING ** buf, DummyCodeRef *dummy)
+PIO_read_async_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+                    NOTNULL(STRING **buf), NOTNULL(DummyCodeRef *dummy))
 {
     while (layer) {
-        if (layer->api->Read_ASync) {
+        if (layer->api->Read_ASync)
             return layer->api->Read_ASync(interp, layer, io, buf, dummy);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return 0;
 }
 
@@ -255,15 +240,13 @@ Returns C<0> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_flush_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io)
+PIO_flush_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io))
 {
     while (layer) {
-        if (layer->api->Flush) {
+        if (layer->api->Flush)
             layer->api->Flush(interp, layer, io);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return 0;
 }
 
@@ -280,16 +263,14 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 PIOOFF_T
-PIO_seek_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io, PIOOFF_T offset,
-              INTVAL whence)
+PIO_seek_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+        PIOOFF_T offset, INTVAL whence)
 {
     while (layer) {
-        if (layer->api->Seek) {
+        if (layer->api->Seek)
             return layer->api->Seek(interp, layer, io, offset, whence);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return -1;
 }
 
@@ -306,15 +287,13 @@ Returns C<0> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 PIOOFF_T
-PIO_tell_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io)
+PIO_tell_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io))
 {
     while (layer) {
-        if (layer->api->Tell) {
+        if (layer->api->Tell)
             return layer->api->Tell(interp, layer, io);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return 0;
 }
 
@@ -331,16 +310,14 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_setbuf_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io,
+PIO_setbuf_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
                 size_t bufsize)
 {
     while (layer) {
-        if (layer->api->SetBuf) {
+        if (layer->api->SetBuf)
             return layer->api->SetBuf(interp, layer, io, bufsize);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return -1;
 }
 
@@ -357,15 +334,13 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_setlinebuf_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io)
+PIO_setlinebuf_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io))
 {
     while (layer) {
-        if (layer->api->SetLineBuf) {
+        if (layer->api->SetLineBuf)
             return layer->api->SetLineBuf(interp, layer, io);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return -1;
 }
 
@@ -382,15 +357,13 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_eof_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), ParrotIO * io)
+PIO_eof_down(PARROT_INTERP, NULLOK(ParrotIOLayer *layer), NOTNULL(ParrotIO *io))
 {
     while (layer) {
-        if (layer->api->Eof) {
+        if (layer->api->Eof)
             return layer->api->Eof(interp, layer, io);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
-    /* No layer found */
     return -1;
 }
 
@@ -407,13 +380,12 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_poll_down(PARROT_INTERP, ParrotIOLayer *layer, ParrotIO *io,
+PIO_poll_down(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
         INTVAL which, INTVAL sec, INTVAL usec)
 {
     while (layer) {
-        if (layer->api->Poll) {
+        if (layer->api->Poll)
             return layer->api->Poll(interp, layer, io, which, sec, usec);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
     return -1;
@@ -431,14 +403,14 @@ Returns C<NULL> if no implementation is found.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO *
-PIO_socket_down(PARROT_INTERP, ParrotIOLayer *layer, INTVAL fam, INTVAL type,
+PIO_socket_down(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), INTVAL fam, INTVAL type,
                 INTVAL proto)
 {
     while (layer) {
-        if (layer->api->Socket) {
+        if (layer->api->Socket)
             return layer->api->Socket(interp, layer, fam, type, proto);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
     return NULL;
@@ -457,12 +429,12 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_recv_down(PARROT_INTERP, ParrotIOLayer *layer, ParrotIO *io, STRING **buf)
+PIO_recv_down(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+        NOTNULL(STRING **buf))
 {
     while (layer) {
-        if (layer->api->Recv) {
+        if (layer->api->Recv)
             return layer->api->Recv(interp, layer, io, buf);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
     return -1;
@@ -481,12 +453,12 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_send_down(PARROT_INTERP, ParrotIOLayer *layer, ParrotIO *io, STRING *buf)
+PIO_send_down(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+        NOTNULL(STRING *buf))
 {
     while (layer) {
-        if (layer->api->Send) {
+        if (layer->api->Send)
             return layer->api->Send(interp, layer, io, buf);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
     return -1;
@@ -505,12 +477,12 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_connect_down(PARROT_INTERP, ParrotIOLayer *layer, ParrotIO *io, STRING *address)
+PIO_connect_down(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+        NOTNULL(STRING *address))
 {
     while (layer) {
-        if (layer->api->Connect) {
+        if (layer->api->Connect)
             return layer->api->Connect(interp, layer, io, address);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
     return -1;
@@ -529,12 +501,12 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_bind_down(PARROT_INTERP, ParrotIOLayer *layer, ParrotIO *io, STRING *address)
+PIO_bind_down(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), NOTNULL(ParrotIO *io),
+        NOTNULL(STRING *address))
 {
     while (layer) {
-        if (layer->api->Bind) {
+        if (layer->api->Bind)
             return layer->api->Bind(interp, layer, io, address);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
     return -1;
@@ -553,12 +525,11 @@ Returns C<-1> if no implementation is found.
 
 PARROT_WARN_UNUSED_RESULT
 INTVAL
-PIO_listen_down(PARROT_INTERP, ParrotIOLayer *layer, ParrotIO *io, INTVAL backlog)
+PIO_listen_down(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), NOTNULL(ParrotIO *io), INTVAL backlog)
 {
     while (layer) {
-        if (layer->api->Listen) {
+        if (layer->api->Listen)
             return layer->api->Listen(interp, layer, io, backlog);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
     return -1;
@@ -576,13 +547,13 @@ Returns C<-1> if no implementation is found.
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 ParrotIO *
-PIO_accept_down(PARROT_INTERP, ParrotIOLayer *layer, ParrotIO *io)
+PIO_accept_down(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer), NOTNULL(ParrotIO *io))
 {
     while (layer) {
-        if (layer->api->Accept) {
+        if (layer->api->Accept)
             return layer->api->Accept(interp, layer, io);
-        }
         layer = PIO_DOWNLAYER(layer);
     }
     return NULL;
