@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: io_stdio.c 19915 2007-07-16 22:08:22Z petdance $
+$Id: io_stdio.c 20342 2007-07-30 05:57:58Z petdance $
 
 =head1 NAME
 
@@ -25,21 +25,24 @@ Win32), this is I<buffered> IO, out of necessity.
 /* HEADERIZER BEGIN: static */
 
 PARROT_CONST_FUNCTION
+PARROT_CANNOT_RETURN_NULL
 static const char * flags_to_stdio( INTVAL flags );
 
 static INTVAL PIO_stdio_close( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io) )
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static ParrotIO * PIO_stdio_fdopen( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     PIOHANDLE fptr,
     INTVAL flags )
         __attribute__nonnull__(1);
 
 static INTVAL PIO_stdio_flush( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io) )
         __attribute__nonnull__(3);
 
@@ -48,15 +51,17 @@ static INTVAL PIO_stdio_init( PARROT_INTERP, NOTNULL(ParrotIOLayer *layer) )
         __attribute__nonnull__(2);
 
 static INTVAL PIO_stdio_isatty( PIOHANDLE fptr );
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static ParrotIO * PIO_stdio_open( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(const char *spath),
     INTVAL flags )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
 static size_t PIO_stdio_peek( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NOTNULL(STRING **buf) )
         __attribute__nonnull__(1)
@@ -64,7 +69,7 @@ static size_t PIO_stdio_peek( PARROT_INTERP,
         __attribute__nonnull__(4);
 
 static size_t PIO_stdio_read( PARROT_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NOTNULL(STRING **buf) )
         __attribute__nonnull__(1)
@@ -72,19 +77,19 @@ static size_t PIO_stdio_read( PARROT_INTERP,
         __attribute__nonnull__(4);
 
 static PIOOFF_T PIO_stdio_seek( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     PIOOFF_T offset,
     INTVAL whence )
         __attribute__nonnull__(3);
 
 static PIOOFF_T PIO_stdio_tell( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io) )
         __attribute__nonnull__(3);
 
 static size_t PIO_stdio_write( SHIM_INTERP,
-    ParrotIOLayer *layer,
+    SHIM(ParrotIOLayer *layer),
     NOTNULL(ParrotIO *io),
     NOTNULL(STRING *s) )
         __attribute__nonnull__(3)
@@ -116,6 +121,7 @@ C<fopen()> in C<PIO_stdio_open()>.
 */
 
 PARROT_CONST_FUNCTION
+PARROT_CANNOT_RETURN_NULL
 static const char *
 flags_to_stdio(INTVAL flags)
 {
@@ -183,6 +189,8 @@ using C<< < >>, C<< > >>, etc.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static ParrotIO *
 PIO_stdio_open(PARROT_INTERP, SHIM(ParrotIOLayer *layer),
               NOTNULL(const char *spath), INTVAL flags)
@@ -220,6 +228,8 @@ PIO_stdio_open(PARROT_INTERP, SHIM(ParrotIOLayer *layer),
 }
 
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static ParrotIO *
 PIO_stdio_fdopen(PARROT_INTERP, SHIM(ParrotIOLayer *layer), PIOHANDLE fptr, INTVAL flags)
 {
