@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: io_utf8.c 19915 2007-07-16 22:08:22Z petdance $
+$Id: io_utf8.c 20694 2007-08-19 01:37:57Z chromatic $
 
 =head1 NAME
 
@@ -86,6 +86,8 @@ ParrotIOLayer pio_utf8_layer = {
     NULL, NULL
 };
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 ParrotIOLayer *
 PIO_utf8_register_layer(void)
 {
@@ -99,33 +101,42 @@ PIO_utf8_read(PARROT_INTERP, NOTNULL(ParrotIOLayer *layer),
     STRING *s, *s2;
     String_iter iter;
 
-    size_t len = PIO_read_down(interp, layer->down, io, buf);
-
-    s = *buf;
+    size_t len  = PIO_read_down(interp, layer->down, io, buf);
+    s           = *buf;
     s->charset  = Parrot_unicode_charset_ptr;
     s->encoding = Parrot_utf8_encoding_ptr;
+
     /* count chars, verify utf8 */
     Parrot_utf8_encoding_ptr->iter_init(interp, s, &iter);
+
     while (iter.bytepos < s->bufused) {
         if (iter.bytepos + 4 > s->bufused) {
             const utf8_t *u8ptr = (utf8_t *)((char *)s->strstart +
                     iter.bytepos);
             UINTVAL c = *u8ptr;
+
             if (UTF8_IS_START(c)) {
                 UINTVAL len2 = UTF8SKIP(u8ptr);
+                INTVAL  read;
+
                 if (iter.bytepos + len2 <= s->bufused)
                     goto ok;
-                /* need len-1 more chars */
+
+                /* need len - 1 more chars */
                 len2--;
-                s2 = NULL;
-                s2 = PIO_make_io_string(interp, &s2, len2);
-                s2->bufused = len2;
+                s2           = NULL;
+                s2           = PIO_make_io_string(interp, &s2, len2);
+                s2->bufused  = len2;
                 s2->charset  = Parrot_unicode_charset_ptr;
                 s2->encoding = Parrot_utf8_encoding_ptr;
-                PIO_read_down(interp, layer->down, io, &s2);
-                s->strlen = iter.charpos;
-                s = string_append(interp, s, s2);
-                len += len2 + 1;
+
+                /* need to check the amount read here? */
+                read         = PIO_read_down(interp, layer->down, io, &s2);
+
+                s->strlen    = iter.charpos;
+                s            = string_append(interp, s, s2);
+                len         += len2 + 1;
+
                 /* check last char */
             }
         }
