@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2006, The Perl Foundation.
-$Id: jit_debug.c 19834 2007-07-14 00:25:36Z coke $
+$Id: jit_debug.c 20350 2007-07-30 15:15:17Z particle $
 
 =head1 NAME
 
@@ -28,6 +28,9 @@ http://sources.redhat.com/gdb/current/onlinedocs/stabs_toc.html.
 
 */
 
+/* HEADERIZER HFILE: none */
+/* HEADERIZER STOP */
+
 #include <parrot/parrot.h>
 #include "parrot/exec.h"
 #include "jit.h"
