@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2002-2007, The Perl Foundation.
 License:  Artistic/GPL, see README and LICENSES for details
-$Id: list.c 20709 2007-08-19 07:03:58Z chromatic $
+$Id: list.c 20926 2007-08-30 21:25:32Z chromatic $
 
 =head1 NAME
 
@@ -69,7 +69,7 @@ be created.
 To avoid this - and the performance penalty - set the array size
 before setting elements.
 
-    new P0, .Array
+    new P0, 'Array'
     set P0, 100000  # sets fixed sized, no sparse
 
 This is only meaningful, if a lot of the entries are used too.
@@ -95,7 +95,7 @@ range. See C<get_chunk> below for details.
 
 To save memory, List can handle sparse arrays. This code snippet:
 
-new P0, .IntList
+new P0, 'IntList'
 set P0[1000000], 42
 
 generates 3 List_chunks, one at the beginning of the array, a
