@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2007, The Perl Foundation
-$Id: main.c 19674 2007-07-07 16:36:54Z paultcochrane $
+$Id: main.c 20576 2007-08-09 14:27:26Z particle $
 
 =head1 NAME
 
@@ -44,10 +44,8 @@ main(int argc, char * argv[])
 
     Parrot_set_config_hash();
 
-    interp     = Parrot_new(NULL);
-    if (!imcc_initialize(interp)) {
-        real_exception(interp, NULL, 1, "Could not initialize IMCC\n");
-    }
+    interp = Parrot_new(NULL);
+    imcc_initialize(interp);
 
     /* We parse the arguments, but first store away the name of the Parrot
        executable, since parsing destroys that and we want to make it
@@ -64,8 +62,6 @@ main(int argc, char * argv[])
     /* Clean-up after ourselves */
     Parrot_destroy(interp);
     Parrot_exit(interp, 0);
-
-    return status;
 }
 
 /*
