@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2006-2007, The Perl Foundation.
-$Id: pic_jit.c 19886 2007-07-15 06:19:53Z petdance $
+$Id: pic_jit.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -29,7 +29,6 @@ TODO:
 
 #include "parrot/parrot.h"
 #include "parrot/oplib/ops.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/pic.h */
 
@@ -73,8 +72,9 @@ static int ops_jittable( PARROT_INTERP,
         __attribute__nonnull__(7);
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static opcode_t * pic_test_func( PARROT_INTERP,
-    INTVAL *sig_bits,
+    SHIM(INTVAL *sig_bits),
     NOTNULL(void **args) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
@@ -97,7 +97,7 @@ static int returns_match_results(
 #  include "parrot/exec.h"
 #  include "jit.h"
 
-#  if PIC_TEST
+#  ifdef PIC_TEST
 /*
  * just for testing the whole scheme ...
 
@@ -120,6 +120,7 @@ static int returns_match_results(
 */
 
 PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static opcode_t *
 pic_test_func(PARROT_INTERP, SHIM(INTVAL *sig_bits), NOTNULL(void **args))
 {
@@ -333,9 +334,10 @@ op_is_ok:
 #endif     /* HAS_JIT */
 
 
+PARROT_WARN_UNUSED_RESULT
 int
 parrot_pic_is_safe_to_jit(PARROT_INTERP, NOTNULL(PMC *sub),
-        PMC *sig_args, PMC *sig_results, int *flags)
+        NOTNULL(PMC *sig_args), NOTNULL(PMC *sig_results), NOTNULL(int *flags))
 {
 #ifdef HAS_JIT
     opcode_t *base, *start, *end;
@@ -390,7 +392,7 @@ funcptr_t
 parrot_pic_JIT_sub(PARROT_INTERP, NOTNULL(PMC *sub), int flags)
 {
 #ifdef HAS_JIT
-#  if PIC_TEST
+#  ifdef PIC_TEST
     UNUSED(interp);
     UNUSED(sub);
     return (funcptr_t) pic_test_func;
