@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: pmc_freeze.c 20709 2007-08-19 07:03:58Z chromatic $
+$Id: pmc_freeze.c 21255 2007-09-13 06:52:30Z chromatic $
 
 =head1 NAME
 
@@ -619,14 +619,15 @@ Pushes the number C<v> onto the end of the C<*io> "stream".
 static void
 push_opcode_number(PARROT_INTERP, NOTNULL(IMAGE_IO *io), FLOATVAL v)
 {
-    const size_t len = PF_size_number() * sizeof (opcode_t);
-    STRING * const s = io->image;
-    const size_t used = s->bufused;
+    const size_t   len  = PF_size_number() * sizeof (opcode_t);
+    STRING * const s    = io->image;
+    const size_t   used = s->bufused;
 
     op_check_size(interp, s, len);
     PF_store_number((opcode_t *)((ptrcast_t)s->strstart + used), &v);
+
     s->bufused += len;
-    s->strlen += len;
+    s->strlen  += len;
 }
 
 /*
@@ -922,7 +923,7 @@ freeze_pmc(PARROT_INTERP, NULLOK(PMC *pmc), NOTNULL(visit_info *info),
 
     if (PMC_IS_NULL(pmc)) {
         /* NULL + seen bit */
-        io->vtable->push_pmc(interp, io, (PMC*) 1);
+        VTABLE_push_pmc(interp, io, (PMC*) 1);
         return;
     }
     type = pmc->vtable->base_type;
@@ -932,8 +933,8 @@ freeze_pmc(PARROT_INTERP, NULLOK(PMC *pmc), NOTNULL(visit_info *info),
     if (seen) {
         if (info->extra_flags) {
             id |= 3;
-            io->vtable->push_pmc(interp, io, (PMC*)id);
-            io->vtable->push_integer(interp, io, info->extra_flags);
+            VTABLE_push_pmc(interp, io, (PMC*)id);
+            VTABLE_push_integer(interp, io, info->extra_flags);
             return;
         }
         id |= 1;         /* mark bit 0 if this PMC is known */
@@ -941,9 +942,9 @@ freeze_pmc(PARROT_INTERP, NULLOK(PMC *pmc), NOTNULL(visit_info *info),
     else if (type == info->last_type) {
         id |= 2;         /* mark bit 1 and don't write type */
     }
-    io->vtable->push_pmc(interp, io, (PMC*)id);
+    VTABLE_push_pmc(interp, io, (PMC*)id);
     if (! (id & 3)) {    /* else write type */
-        io->vtable->push_integer(interp, io, type);
+        VTABLE_push_integer(interp, io, type);
         info->last_type = type;
     }
 }
@@ -978,10 +979,10 @@ thaw_pmc(PARROT_INTERP, NOTNULL(visit_info *info),
     int seen = 0;
 
     info->extra_flags = EXTRA_IS_NULL;
-    n = io->vtable->shift_pmc(interp, io);
+    n = VTABLE_shift_pmc(interp, io);
     if (((UINTVAL) n & 3) == 3) {
         /* pmc has extra data */
-        info->extra_flags = io->vtable->shift_integer(interp, io);
+        info->extra_flags = VTABLE_shift_integer(interp, io);
     }
     else if ((UINTVAL) n & 1) {     /* seen PMCs have bit 0 set */
         seen = 1;
@@ -990,7 +991,7 @@ thaw_pmc(PARROT_INTERP, NOTNULL(visit_info *info),
         *type = info->last_type;
     }
     else {                       /* type follows */
-        info->last_type = *type = io->vtable->shift_integer(interp, io);
+        info->last_type = *type = VTABLE_shift_integer(interp, io);
         if (*type <= 0)
             real_exception(interp, NULL, 1, "Unknown PMC type to thaw %d", (int) *type);
         if (*type >= interp->n_vtable_max ||
@@ -1425,6 +1426,10 @@ visit_loop_todo_list(PARROT_INTERP, NULLOK(PMC *current),
 again:
     for (; (int)list_length(interp, todo);) {
         current = *(PMC**)list_shift(interp, todo, enum_type_PMC);
+        if (!current) {
+            real_exception(interp, NULL, 1,
+                    "NULL current PMC in visit_loop_todo_list");
+        }
         VTABLE_visit(interp, current, info);
         if (thawing) {
             if (current == info->thaw_result)
