@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2003, The Perl Foundation.
-$Id: spf_vtable.c 19833 2007-07-13 22:02:25Z petdance $
+$Id: spf_vtable.c 20709 2007-08-19 07:03:58Z chromatic $
 
 =head1 NAME
 
@@ -26,72 +26,90 @@ retrieve arguments.
 
 /* HEADERIZER BEGIN: static */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static STRING * getchr_pmc( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static STRING * getchr_va( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
 static HUGEFLOATVAL getfloat_pmc( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
 static HUGEFLOATVAL getfloat_va( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
 static HUGEINTVAL getint_pmc( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
 static HUGEINTVAL getint_va( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static void * getptr_pmc( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static void * getptr_va( PARROT_INTERP,
-    INTVAL size,
+    SHIM(INTVAL size),
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING * getstring_pmc( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING * getstring_va( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
 static UHUGEINTVAL getuint_pmc( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
         __attribute__nonnull__(1)
         __attribute__nonnull__(3);
 
+PARROT_WARN_UNUSED_RESULT
 static UHUGEINTVAL getuint_va( PARROT_INTERP,
     INTVAL size,
     NOTNULL(SPRINTF_OBJ *obj) )
@@ -111,6 +129,8 @@ C<size> is unused.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static STRING *
 getchr_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -134,6 +154,7 @@ of the integer.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 static HUGEINTVAL
 getint_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -165,7 +186,6 @@ getint_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
         }
     default:
         PANIC(interp, "Invalid int type!");
-        return 0;
     }
 }
 
@@ -181,6 +201,7 @@ of the integer.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 static UHUGEINTVAL
 getuint_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -212,7 +233,6 @@ getuint_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
         }
     default:
         PANIC(interp, "Invalid uint type!");
-        return 0;
     }
 }
 
@@ -228,6 +248,7 @@ the number.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 static HUGEFLOATVAL
 getfloat_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -274,6 +295,8 @@ of the string.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING *
 getstring_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -306,7 +329,6 @@ getstring_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
         real_exception(interp, NULL, INVALID_CHARACTER,
                 "Internal sprintf doesn't recognize size %d for a string",
                 size);
-        return NULL;
     }
 }
 
@@ -320,8 +342,10 @@ C<size> is unused.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
 static void *
-getptr_va(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
+getptr_va(PARROT_INTERP, SHIM(INTVAL size), NOTNULL(SPRINTF_OBJ *obj))
 {
     va_list * const arg = (va_list *)(obj->data);
 
@@ -344,6 +368,8 @@ from C<obj>.
 
 */
 
+PARROT_CANNOT_RETURN_NULL
+PARROT_WARN_UNUSED_RESULT
 static STRING *
 getchr_pmc(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -367,6 +393,7 @@ from C<obj>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 static HUGEINTVAL
 getint_pmc(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -402,6 +429,7 @@ from C<obj>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 static UHUGEINTVAL
 getuint_pmc(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -437,6 +465,7 @@ from C<obj>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
 static HUGEFLOATVAL
 getfloat_pmc(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -469,6 +498,8 @@ from C<obj>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static STRING *
 getstring_pmc(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
@@ -491,16 +522,19 @@ from C<obj>.
 
 */
 
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 static void *
 getptr_pmc(PARROT_INTERP, INTVAL size, NOTNULL(SPRINTF_OBJ *obj))
 {
     PMC * const tmp = VTABLE_get_pmc_keyed_int(interp,
-            ((PMC *)obj->data),
-            (obj->index));
+            ((PMC *)obj->data), (obj->index));
+    INTVAL      i   = VTABLE_get_integer(interp, tmp);
 
     obj->index++;
+
     /* XXX correct? */
-    return (void *)(VTABLE_get_integer(interp, tmp));
+    return (void *)i;
 }
 
 SPRINTF_OBJ pmc_core = {
