@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: stack_common.c 19816 2007-07-12 15:39:05Z petdance $
+$Id: stack_common.c 20247 2007-07-27 02:22:36Z petdance $
 
 =head1 NAME
 
@@ -19,7 +19,6 @@ These stacks all differ only in the size of items.
 
 #include "parrot/parrot.h"
 #include "parrot/stacks.h"
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/stacks.h */
 
@@ -48,6 +47,7 @@ debugging/error reporting.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 Stack_Chunk_t *
 register_new_stack(PARROT_INTERP, NOTNULL(const char *name), size_t item_size)
 {
@@ -71,6 +71,7 @@ Get a new chunk either from the freelist or allocate one.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 Stack_Chunk_t *
 cst_new_stack_chunk(PARROT_INTERP, NOTNULL(const Stack_Chunk_t *chunk))
 {
@@ -95,6 +96,7 @@ Return a pointer, where new entries go for push.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 void*
 stack_prepare_push(PARROT_INTERP, NOTNULL(Stack_Chunk_t **stack_p))
 {
@@ -116,6 +118,7 @@ Return a pointer, where new entries are popped off.
 
 PARROT_API
 PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
 void*
 stack_prepare_pop(PARROT_INTERP, NOTNULL(Stack_Chunk_t **stack_p))
 {
