@@ -1,3 +1,8 @@
+/*
+ * $Id: backend.c 21226 2007-09-12 18:57:36Z paultcochrane $
+ * Copyright (C) 2006-2007, The Perl Foundation
+ */
+
 #include <parrot/parrot.h>
 #include <parrot/atomic.h>
 
@@ -242,13 +247,13 @@ void
 Parrot_freeze_STM_PMC_handle(PARROT_INTERP, NOTNULL(IMAGE_IO *io),
             NOTNULL(Parrot_STM_PMC_handle handle))
 {
-    io->vtable->push_pmc(interp, io, (PMC*) handle);
+    VTABLE_push_pmc(interp, io, (PMC*) handle);
 }
 
 Parrot_STM_PMC_handle
 Parrot_thaw_STM_PMC_handle(PARROT_INTERP, NOTNULL(IMAGE_IO *io))
 {
-    Parrot_STM_PMC_handle handle = (Parrot_STM_PMC_handle) io->vtable->shift_pmc(interp, io);
+    Parrot_STM_PMC_handle handle = (Parrot_STM_PMC_handle) VTABLE_shift_pmc(interp, io);
     return handle;
 }
 
