@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2001-2007, The Perl Foundation.
-$Id: warnings.c 19806 2007-07-12 04:26:28Z petdance $
+$Id: warnings.c 20241 2007-07-27 02:03:48Z petdance $
 
 =head1 NAME
 
@@ -16,7 +16,6 @@ messages.
 #include "parrot/parrot.h"
 
 #include <stdarg.h>
-#include <assert.h>
 
 /* HEADERIZER HFILE: include/parrot/warnings.h */
 
@@ -91,7 +90,7 @@ Parrot_warn(PARROT_INTERP, INTVAL warnclass,
 
     va_list args;
 
-    assert(interp);
+    PARROT_ASSERT(interp);
     if (!PARROT_WARNINGS_test(interp, warnclass))
         return 2;
 
