@@ -1,7 +1,7 @@
 #! perl
 ################################################################################
 # Copyright (C) 2001-2003, The Perl Foundation.
-# $Id: /parrotcode/local/tools/dev/lib_deps.pl 733 2006-12-17T23:24:17.491923Z chromatic  $
+# $Id: lib_deps.pl 18810 2007-06-04 20:27:35Z paultcochrane $
 ################################################################################
 
 =head1 NAME
@@ -101,7 +101,7 @@ sub do_source {
     print "Running cxref (pass 1)\n";
     system("$cmd > $devnull 2>$devnull");
     print "Running cxref (pass 2)\n";
-    open( F, '<', "$cmd 2>$devnull|" )
+    open( my $F, '<', "$cmd 2>$devnull|" )
         || die "Can't run $cmd.\n";
 
     my %external_calls;
@@ -109,7 +109,7 @@ sub do_source {
     my %variable_visible;
     my %system_include;
     my ( $file, $function, $variable );
-    while (<F>) {
+    while (<$F>) {
 
         if (/----------------------------------------/) {
             undef $file if defined($file);
@@ -182,7 +182,7 @@ sub do_source {
         }
     }
 
-    close(F);
+    close($F);
 
     # filter out things that start with _.  Probably internal libc stuff.
     my @external_calls          = grep { !/^_/ } sort keys %external_calls;
@@ -215,9 +215,9 @@ sub do_source {
 
 sub do_object {
     foreach my $obj (@files) {
-        open( F, '<', "nm -a $obj|" ) || die "Can't run nm on $obj\n";
+        open( my $F, '<', "nm -a $obj|" ) || die "Can't run nm on $obj\n";
 
-        while (<F>) {
+        while (<$F>) {
             chomp;
 
             my ( $type, $symbol ) = /^........ (\S) (.*)/;
@@ -231,7 +231,7 @@ sub do_object {
             }
         }
 
-        close(F);
+        close($F);
     }
 
     # omit symbols which begin with _.  These are likely to be internal
