@@ -1,6 +1,6 @@
 #! perl
-# Copyright (C) 2004, The Perl Foundation.
-# $Id: parrot_api.pl 18810 2007-06-04 20:27:35Z paultcochrane $
+# Copyright (C) 2004-2007, The Perl Foundation.
+# $Id: parrot_api.pl 19262 2007-06-22 20:17:11Z bernhard $
 
 =head1 NAME
 
@@ -175,7 +175,7 @@ my %DataR;
 my %Undef;
 my %API;
 
-if ( open( my $NM, "perl tools/dev/nm.pl -BDo '$Obj' |" ) ) {
+if ( open( my $NM, '<', "perl tools/dev/nm.pl -BDo '$Obj' |" ) ) {
     while (<$NM>) {
         my ( $o, $s, $v ) = split;
         $API{$s} = $o;
